#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.
#
'''Listener v3 action implementations'''
import logging

from osc_lib import utils
from osc_lib.cli import parseractions
from osc_lib.command import command

from otcextensions.i18n import _

LOG = logging.getLogger(__name__)


def _flatten_listener(obj):
    """Flatten the structure of the listener into a single dict
    """
    data = {
        'client_ca_tls_container_ref': obj.client_ca_tls_container_ref,
        'connection_limit': obj.connection_limit,
        'created_at': obj.created_at,
        'default_pool_id': obj.default_pool_id,
        'default_tls_container_ref': obj.default_tls_container_ref,
        'description': obj.description,
        'http2_enable': obj.http2_enable,
        'id': obj.id,
        'insert_headers': obj.insert_headers,
        'is_admin_state_up': obj.is_admin_state_up,
        'name': obj.name,
        'project_id': obj.project_id,
        'security_policy_id': obj.security_policy_id,
        'sni_match_algo': obj.sni_match_algo,
        'protocol': obj.protocol,
        'protocol_port': obj.protocol_port,
        'sni_container_refs': obj.sni_container_refs,
        'updated_at': obj.updated_at,
        'tls_ciphers_policy': obj.tls_ciphers_policy,
        'enable_member_retry': obj.enable_member_retry,
        'keepalive_timeout': obj.keepalive_timeout,
        'client_timeout': obj.client_timeout,
        'member_timeout': obj.member_timeout,
        'ipgroup': obj.ipgroup,
        'transparent_client_ip_enable': obj.transparent_client_ip_enable,
        'enhance_l7policy_enable': obj.enhance_l7policy
    }
    return data


def _add_loadbalancers_to_listener_obj(obj, data, columns):
    """Add associated loadbalancers to column and data tuples
    """
    i = 0
    for lb in obj.load_balancers:
        name = 'loadbalancer_id_' + str(i + 1)
        data += (lb['id'],)
        columns = columns + (name,)
        i += 1
    return data, columns


def _add_tags_to_listener_obj(obj, data, columns):
    data += ('\n'.join((f'value={tag["value"]}, key={tag["key"]}'
                        for tag in obj.tags)),)
    columns = columns + ('tags',)
    return data, columns


def _normalize_tags(tags):
    result = []
    for tag in tags:
        try:
            tag = tag.split('=')
            result.append({
                'key': tag[0],
                'value': tag[1]
            })
        except IndexError:
            result.append({'key': tag[0], 'value': ''})
    return result


class ListListeners(command.Lister):
    _description = _('List listeners')
    columns = ('ID', 'Name', 'provisioning_status')

    def get_parser(self, prog_name):
        parser = super(ListListeners, self).get_parser(prog_name)
        parser.add_argument(
            '--id',
            help=_('Specifies the listener ID.')
        )
        parser.add_argument(
            '--name',
            metavar='<name>',
            help=_('Specifies the name of the listener added'
                   'to the load balancer.')
        )
        parser.add_argument(
            '--protocol-port',
            metavar='protocol_port',
            help=_('Specifies the port used by the listener.')
        )
        parser.add_argument(
            '--protocol',
            help=_('Specifies the protocol used by the listener.'
                   'The protocol can be UDP, TCP, HTTP, or HTTPS.'
                   'Repeat for multiple values.')
        )
        parser.add_argument(
            '--description',
            help=_('Provides supplementary information about the listener.')
        )
        parser.add_argument(
            '--default-tls-container-ref',
            metavar='default_tls_container_ref',
            help=_('Specifies the ID of the server certificate used'
                   'by the listener.')
        )
        parser.add_argument(
            '--client-ca-tls-container-ref',
            metavar='<client_ca_tls_container_ref>',
            help=_('Limit the number of results displayed')
        )
        parser.add_argument(
            '--default-pool-id',
            metavar='default_pool_id',
            help=_('Specifies the ID of the default backend server group.')
        )
        parser.add_argument(
            '--http2-enable',
            metavar='<http2_enable>',
            type=bool,
            help=_('Specifies whether to use HTTP/2. This parameter'
                   'is available only for HTTPS listeners. If you configure'
                   'this parameter for other types of listeners,'
                   'it will not take effect.')
        )
        parser.add_argument(
            '--loadbalancer-id',
            metavar='loadbalancer_id',
            help=_('Specifies the ID of the load balancer'
                   'that the listener is added to.')
        )
        parser.add_argument(
            '--tls-ciphers-policy',
            metavar='<tls_ciphers_policy>',
            help=_('Specifies the security policy used by the listener.'
                   'This parameter is available only for HTTPS listeners.')
        )
        parser.add_argument(
            '--member-address',
            metavar='<member_address>',
            help=_('Specifies the private IP address bound'
                   'to the backend server.')
        )
        parser.add_argument(
            '--member-device-id',
            metavar='<member_device_id>',
            help=_('Specifies the ID of the cloud server that serves'
                   'as a backend server.')
        )
        parser.add_argument(
            '--enable-member-retry',
            metavar='<enable_member_retry>',
            type=bool,
            help=_('Specifies whether to enable health check retries'
                   'for backend servers.')
        )
        parser.add_argument(
            '--member-timeout',
            metavar='<member_timeout>',
            help=_('Specifies the timeout duration for waiting for a request'
                   'from a backend server, in seconds.')
        )
        parser.add_argument(
            '--client-timeout',
            metavar='<client_timeout>',
            help=_('Specifies the timeout duration for waiting for a request'
                   'from a client, in seconds.')
        )
        parser.add_argument(
            '--keepalive-timeout',
            metavar='<keepalive_timeout>',
            help=_('Specifies the idle timeout duration, in seconds.')
        )
        parser.add_argument(
            '--transparent-client-ip-enable',
            metavar='<transparent_client_ip_enable>',
            type=bool,
            help=_('Specifies whether to pass source IP addresses'
                   'of the clients to backend servers.')
        )
        parser.add_argument(
            '--limit',
            type=int,
            help=_('Specifies the number of records on each page.')
        )
        parser.add_argument(
            '--marker',
            help=_('Specifies the ID of the last record on the previous page.'
                   'This parameter must be used together with limit.')
        )
        parser.add_argument(
            '--page-reverse',
            metavar='page_reverse',
            type=bool,
            help=_('Specifies the page direction.')
        )
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.vlb

        args = {}
        if parsed_args.id:
            args['id'] = parsed_args.id
        if parsed_args.name:
            args['name'] = parsed_args.name
        if parsed_args.protocol_port:
            args['protocol_port'] = parsed_args.protocol_port
        if parsed_args.protocol:
            args['protocol'] = parsed_args.protocol
        if parsed_args.description:
            args['description'] = parsed_args.description
        if parsed_args.default_tls_container_ref:
            args['default_tls_container_ref'] =\
                parsed_args.default_tls_container_ref
        if parsed_args.client_ca_tls_container_ref:
            args['client_ca_tls_container_ref'] =\
                parsed_args.client_ca_tls_container_ref
        if parsed_args.default_pool_id:
            args['default_pool_id'] = parsed_args.default_pool_id
        if parsed_args.http2_enable is not None:
            args['http2_enable'] = parsed_args.http2_enable
        if parsed_args.loadbalancer_id:
            args['loadbalancer_id'] = parsed_args.loadbalancer_id
        if parsed_args.tls_ciphers_policy:
            args['tls_ciphers_policy'] = parsed_args.tls_ciphers_policy
        if parsed_args.member_address:
            args['member_address'] = parsed_args.member_address
        if parsed_args.member_device_id:
            args['member_device_id'] = parsed_args.member_device_id
        if parsed_args.enable_member_retry is not None:
            args['enable_member_retry'] = parsed_args.enable_member_retry
        if parsed_args.member_timeout:
            args['member_timeout'] = parsed_args.member_timeout
        if parsed_args.client_timeout:
            args['client_timeout'] = parsed_args.client_timeout
        if parsed_args.keepalive_timeout:
            args['keepalive_timeout'] = parsed_args.keepalive_timeout
        if parsed_args.transparent_client_ip_enable:
            args['transparent_client_ip_enable'] =\
                parsed_args.transparent_client_ip_enable
        if parsed_args.limit:
            args['limit'] = parsed_args.limit
        if parsed_args.marker:
            args['marker'] = parsed_args.marker
        if parsed_args.page_reverse:
            args['page_reverse'] = parsed_args.page_reverse
        data = client.listeners(**args)

        return (
            self.columns,
            (utils.get_item_properties(
                s, self.columns,
            ) for s in data)
        )


class ShowListener(command.ShowOne):
    _description = _('Shows details of a listener')
    columns = (
        'ID',
        'name',
        'client_ca_tls_container_ref',
        'provisioning_status',
        'connection_limit',
        'created_at',
        'default_pool_id',
        'default_tls_container_ref',
        'description',
        'http2_enable',
        'project_id',
        'protocol',
        'protocol_port',
        'sni_container_refs',
        'updated_at',
        'tls_ciphers_policy',
        'enable_member_retry',
        'keepalive_timeout',
        'client_timeout',
        'member_timeout',
        'transparent_client_ip_enable'
    )

    def get_parser(self, prog_name):
        parser = super(ShowListener, self).get_parser(prog_name)
        parser.add_argument(
            'listener',
            metavar='<listener>',
            help=_('ID or name of the listener')
        )
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.vlb

        obj = client.find_listener(
            name_or_id=parsed_args.listener,
            ignore_missing=False
        )

        data = utils.get_dict_properties(
            _flatten_listener(obj), self.columns)
        if obj.load_balancers:
            data, self.columns = _add_loadbalancers_to_listener_obj(
                obj, data, self.columns)

        return self.columns, data


class CreateListener(command.ShowOne):
    _description = _('Create listener')
    columns = (
        'ID',
        'name',
        'client_ca_tls_container_ref',
        'provisioning_status',
        'connection_limit',
        'created_at',
        'default_pool_id',
        'default_tls_container_ref',
        'description',
        'http2_enable',
        'project_id',
        'protocol',
        'protocol_port',
        'sni_container_refs',
        'updated_at',
        'tls_ciphers_policy',
        'enable_member_retry',
        'keepalive_timeout',
        'client_timeout',
        'member_timeout',
        'transparent_client_ip_enable'
    )

    def get_parser(self, prog_name):
        parser = super(CreateListener, self).get_parser(prog_name)
        parser.add_argument(
            '--name',
            metavar='<name>',
            help=_('Name of the load balancer')
        )
        parser.add_argument(
            '--client-ca-tls-container-ref',
            metavar='<client_ca_tls_container_ref>',
            help=_('Specifies the ID of the CA certificate'
                   'used by the listener.')
        )
        parser.add_argument(
            '--default-pool-id',
            metavar='<default_pool_id>',
            help=_('Specifies the ID of the default backend server group.'
                   'If there is no matched forwarding policy, requests are'
                   'forwarded to the default backend server for processing.')
        )
        parser.add_argument(
            '--default-tls-container-ref',
            metavar='<default_tls_container_ref>',
            help=_('Specifies the ID of the server certificate'
                   'used by the listener.')
        )
        parser.add_argument(
            '--description',
            type=str,
            help=_('Provides supplementary information about the listener.')
        )
        parser.add_argument(
            '--http2-enable',
            action='store_true',
            help=_('Specifies whether to use HTTP/2. This parameter'
                   'is available only for HTTPS listeners. If you configure'
                   'this parameter for other types of listeners, it will not'
                   'take effect.')
        )
        parser.add_argument(
            '--insert-headers',
            metavar='<insert_headers>',
            type=dict,
            help=_('insert_headers')
        )
        parser.add_argument(
            '--loadbalancer-id',
            metavar='<loadbalancer_id>',
            required=True,
            help=_('Specifies the ID of the load balancer that the listener'
                   'is added to.')
        )
        parser.add_argument(
            '--project-id',
            metavar='project_id',
            help=_('Specifies the project ID.')
        )
        parser.add_argument(
            '--protocol',
            required=True,
            choices=['TCP', 'HTTP', 'UDP', 'HTTPS'],
            help=_('Specifies the protocol used by the listener.')
        )
        parser.add_argument(
            '--protocol-port',
            metavar='<protocol_port>',
            type=int,
            required=True,
            help=_('Specifies the port used by the listener.')
        )
        parser.add_argument(
            '--sni-container-refs',
            action='append',
            dest='sni_container_refs',
            help=_('Lists the IDs of SNI certificates (server certificates'
                   'with domain names) used by the listener.'
                   'This parameter will be ignored and an empty array will be'
                   'returned if the listener protocol is not HTTPS.')
        )
        parser.add_argument(
            '--sni-match-algo',
            metavar='<sni_match_algo>',
            help=_('Specifies how wildcard domain name matches with the SNI'
                   'certificates used by the listener.'
                   'longest_suffix indicates longest suffix match.'
                   'wildcard indicates wildcard match.'
                   'The default value is wildcard.')
        )
        parser.add_argument(
            '--tag',
            metavar='key=<keyname1>,value=<value1>',
            action=parseractions.MultiKeyValueAction,
            dest='tags',
            required_keys=['key', 'value'],
            help=_('List of tags. Repeat option for '
                   'multiple tags.\n'
                   'Example:\n'
                   '--tag key=mykey1,value=myvalue1')
        )
        parser.add_argument(
            '--tls-ciphers-policy',
            metavar='<tls_ciphers_policy>',
            choices=['tls-1-0', 'tls-1-1', 'tls-1-2', 'tls-1-2-strict'],
            help=_('Specifies the security policy that will be used'
                   'by the listener. This parameter is available only for'
                   'HTTPS listeners. The default value is tls-1-0.')
        )
        parser.add_argument(
            '--security-policy-id',
            metavar='<security_policy_id>',
            help=_('Specifies the ID of the custom security policy.')
        )
        parser.add_argument(
            '--disable-member-retry',
            action='store_true',
            help=_('Specifies whether to enable health check retries for'
                   'backend servers. This parameter is available only for'
                   'HTTP and HTTPS listeners.')
        )
        parser.add_argument(
            '--keepalive-timeout',
            metavar='<keepalive_timeout>',
            type=int,
            help=_('Specifies the idle timeout duration, in seconds.')
        )
        parser.add_argument(
            '--client-timeout',
            metavar='<client_timeout>',
            type=int,
            help=_('Specifies the timeout duration for waiting for'
                   'a request from a client, in seconds. This parameter is'
                   'available only for HTTP and HTTPS listeners.'
                   '60 by default.')
        )
        parser.add_argument(
            '--member-timeout',
            metavar='<member_timeout>',
            type=int,
            help=_('Specifies the timeout duration for waiting for a request'
                   'from a backend server, in seconds. This parameter is'
                   'available only for HTTP and HTTPS listeners.'
                   'The value ranges from 1 to 300, and the default'
                   'value is 60.')
        )
        parser.add_argument(
            '--ipgroup-id',
            metavar='<ipgroup_id>',
            help=_('Specifies the ID of the IP address group associated with'
                   'the listener.')
        )
        parser.add_argument(
            '--disable-ipgroup',
            # default enable true
            action='store_true',
            help=_('Specifies whether to enable access control.')
        )
        parser.add_argument(
            '--ipgroup-type',
            metavar='<ipgroup_type>',
            help=_('Specifies how access to the listener is controlled.'
                   'Can be black and white.')
        )
        parser.add_argument(
            '--enable-enhance_l7policy',
            # default false
            action='store_true',
            help=_('Specifies whether to enable advanced forwarding.'
                   'If advanced forwarding is enabled, more flexible'
                   'forwarding policies and rules are supported.')
        )
        return parser

    def take_action(self, parsed_args):
        attrs = {}

        attrs['protocol'] = parsed_args.protocol
        attrs['protocol_port'] = parsed_args.protocol_port
        attrs['loadbalancer_id'] = parsed_args.loadbalancer_id

        if parsed_args.name:
            attrs['name'] = parsed_args.name
        if parsed_args.client_ca_tls_container_ref:
            attrs['client_ca_tls_container_ref'] =\
                parsed_args.client_ca_tls_container_ref
        if parsed_args.default_pool_id:
            attrs['default_pool_id'] = parsed_args.default_pool_id
        if parsed_args.default_tls_container_ref:
            attrs['default_tls_container_ref'] =\
                parsed_args.default_tls_container_ref
        if parsed_args.description:
            attrs['description'] = parsed_args.description
        if parsed_args.http2_enable:
            attrs['http2_enable'] = True
        if parsed_args.insert_headers:
            attrs['insert_headers'] = parsed_args.insert_headers
        if parsed_args.project_id:
            attrs['project_id'] = parsed_args.project_id
        if parsed_args.sni_container_refs:
            attrs['sni_container_refs'] = parsed_args.sni_container_refs
        if parsed_args.sni_match_algo:
            attrs['sni_match_algo'] = parsed_args.sni_match_algo
        if parsed_args.tags:
            attrs['tags'] = parsed_args.tags
        if parsed_args.tls_ciphers_policy:
            attrs['tls_ciphers_policy'] = parsed_args.tls_ciphers_policy
        if parsed_args.security_policy_id:
            attrs['security_policy_id'] = parsed_args.security_policy_id
        if parsed_args.disable_member_retry is True:
            attrs['enable_member_retry'] = False
        if parsed_args.keepalive_timeout:
            attrs['keepalive_timeout'] = parsed_args.keepalive_timeout
        if parsed_args.client_timeout:
            attrs['client_timeout'] = parsed_args.client_timeout
        if parsed_args.member_timeout:
            attrs['member_timeout'] = parsed_args.member_timeout
        if parsed_args.enable_enhance_l7policy:
            attrs['enable_enhance_l7policy'] = True
        if parsed_args.ipgroup_id:
            attrs['ipgroup'] = {'ipgroup_id': parsed_args.ipgroup_id}
            if parsed_args.disable_ipgroup is True:
                attrs['ipgroup']['enable_ipgroup'] = False
            if parsed_args.ipgroup_type:
                attrs['ipgroup']['type'] = parsed_args.ipgroup_type

        client = self.app.client_manager.vlb
        obj = client.create_listener(**attrs)

        data = utils.get_dict_properties(
            _flatten_listener(obj), self.columns)

        if obj.tags:
            data, self.columns = _add_tags_to_listener_obj(
                obj, data, self.columns)
        if obj.load_balancers:
            data, self.columns = _add_loadbalancers_to_listener_obj(
                obj, data, self.columns)
        return self.columns, data


class UpdateListener(command.ShowOne):
    _description = _('Update listener')
    columns = (
        'ID',
        'name',
        'client_ca_tls_container_ref',
        'connection_limit',
        'created_at',
        'default_pool_id',
        'default_tls_container_ref',
        'description',
        'insert_headers',
        'http2_enable',
        'project_id',
        'protocol',
        'protocol_port',
        'sni_container_refs',
        'updated_at',
        'tls_ciphers_policy',
        'enable_member_retry',
        'keepalive_timeout',
        'client_timeout',
        'member_timeout',
        'transparent_client_ip_enable'
    )

    def get_parser(self, prog_name):
        parser = super(UpdateListener, self).get_parser(prog_name)
        parser.add_argument(
            'listener',
            help=_('Specifies listener name of id.')
        )
        parser.add_argument(
            '--name',
            help=_('Specifies the new name.')
        )
        parser.add_argument(
            '--client-ca-tls-container-ref',
            metavar='<client_ca_tls_container_ref>',
            help=_('Specifies the ID of the CA certificate'
                   'used by the listener.')
        )
        parser.add_argument(
            '--default-pool-id',
            metavar='<default_pool_id>',
            help=_('Specifies the ID of the default backend server group.'
                   'If there is no matched forwarding policy, requests are'
                   'forwarded to the default backend server for processing.')
        )
        parser.add_argument(
            '--default-tls-container-ref',
            metavar='<default_tls_container_ref>',
            help=_('Specifies the ID of the server certificate'
                   'used by the listener.')
        )
        parser.add_argument(
            '--description',
            type=str,
            help=_('Provides supplementary information about the listener.')
        )
        parser.add_argument(
            '--http2-enable',
            type='store_true',
            help=_('Specifies whether to use HTTP/2. This parameter'
                   'is available only for HTTPS listeners. If you configure'
                   'this parameter for other types of listeners, it will not'
                   'take effect.')
        )
        parser.add_argument(
            '--insert-headers',
            metavar='<insert_headers>',
            type=dict,
            help=_('insert_headers')
        )
        parser.add_argument(
            '--sni-container-refs',
            action='append',
            dest='sni_container_refs',
            help=_('Lists the IDs of SNI certificates (server certificates'
                   'with domain names) used by the listener.'
                   'This parameter will be ignored and an empty array will be'
                   'returned if the listener protocol is not HTTPS.')
        )
        parser.add_argument(
            '--sni-match-algo',
            metavar='<sni_match_algo>',
            help=_('Specifies how wildcard domain name matches with the SNI'
                   'certificates used by the listener. longest_suffix '
                   'indicates longest suffix match. wildcard indicates'
                   'wildcard match.')
        )
        parser.add_argument(
            '--tls-ciphers-policy',
            metavar='<tls_ciphers_policy>',
            choices=['tls-1-0', 'tls-1-1', 'tls-1-2', 'tls-1-2-strict'],
            help=_('Specifies the security policy that will be used'
                   'by the listener. This parameter is available only for'
                   'HTTPS listeners. The default value is tls-1-0.')
        )
        parser.add_argument(
            '--security-policy-id',
            metavar='<security_policy_id>',
            help=_('Specifies the ID of the custom security policy.')
        )
        parser.add_argument(
            '--disable-member-retry',
            action='story_true',
            help=_('Specifies whether to enable health check retries for'
                   'backend servers. This parameter is available only for'
                   'HTTP and HTTPS listeners.')
        )
        parser.add_argument(
            '--keepalive-timeout',
            metavar='<keepalive_timeout>',
            type=int,
            help=_('Specifies the idle timeout duration, in seconds.')
        )
        parser.add_argument(
            '--client-timeout',
            metavar='<client_timeout>',
            type=int,
            help=_('Specifies the timeout duration for waiting for'
                   'a request from a client, in seconds. This parameter is'
                   'available only for HTTP and HTTPS listeners.'
                   '60 by default.')
        )
        parser.add_argument(
            '--member-timeout',
            metavar='<member_timeout>',
            type=int,
            help=_('Specifies the timeout duration for waiting for a request'
                   'from a backend server, in seconds. This parameter is'
                   'available only for HTTP and HTTPS listeners.'
                   'The value ranges from 1 to 300, and the default'
                   'value is 60.')
        )
        parser.add_argument(
            '--ipgroup-id',
            metavar='<ipgroup_id>',
            help=_('Specifies the ID of the IP address group associated with'
                   'the listener.')
        )
        parser.add_argument(
            '--disable-ipgroup',
            action='store_true',
            help=_('Specifies whether to enable access control.')
        )
        parser.add_argument(
            '--ipgroup-type',
            metavar='<ipgroup_type>',
            help=_('Specifies how access to the listener is controlled.'
                   'Can be black and white.')
        )
        parser.add_argument(
            '--enable-enhance_l7policy',
            # default false
            action='store_true',
            help=_('Specifies whether to enable advanced forwarding.'
                   'If advanced forwarding is enabled, more flexible'
                   'forwarding policies and rules are supported.')
        )
        return parser

    def take_action(self, parsed_args):
        attrs = {}

        if parsed_args.name:
            attrs['name'] = parsed_args.name
        if parsed_args.client_ca_tls_container_ref:
            attrs['client_ca_tls_container_ref'] =\
                parsed_args.client_ca_tls_container_ref
        if parsed_args.default_pool_id:
            attrs['default_pool_id'] = parsed_args.default_pool_id
        if parsed_args.default_tls_container_ref:
            attrs['default_tls_container_ref'] =\
                parsed_args.default_tls_container_ref
        if parsed_args.description:
            attrs['description'] = parsed_args.description
        if parsed_args.http2_enable is True:
            attrs['http2_enable'] = True
        if parsed_args.insert_headers:
            attrs['insert_headers'] = parsed_args.insert_headers
        if parsed_args.sni_container_refs:
            attrs['sni_container_refs'] = parsed_args.sni_container_refs
        if parsed_args.sni_match_algo:
            attrs['sni_match_algo'] = parsed_args.sni_match_algo
        if parsed_args.tls_ciphers_policy:
            attrs['tls_ciphers_policy'] = parsed_args.tls_ciphers_policy
        if parsed_args.security_policy_id:
            attrs['security_policy_id'] = parsed_args.security_policy_id
        if parsed_args.disable_member_retry is True:
            attrs['enable_member_retry'] = False
        if parsed_args.keepalive_timeout:
            attrs['keepalive_timeout'] = parsed_args.keepalive_timeout
        if parsed_args.client_timeout:
            attrs['client_timeout'] = parsed_args.client_timeout
        if parsed_args.member_timeout:
            attrs['member_timeout'] = parsed_args.member_timeout
        if parsed_args.ipgroup_id or parsed_args.enable_ipgroup or\
                parsed_args.ipgroup_type:
            attrs['ipgroup'] = {}
            if parsed_args.ipgroup_id:
                attrs['ipgroup']['ipgroup_id'] = parsed_args.ipgroup_id
            if parsed_args.disable_ipgroup is True:
                attrs['ipgroup']['enable_ipgroup'] = False
            if parsed_args.ipgroup_type:
                attrs['ipgroup']['type'] = parsed_args.ipgroup_type

        client = self.app.client_manager.vlb

        listener = client.find_listener(
            name_or_id=parsed_args.listener,
            ignore_missing=False
        )
        obj = client.update_listener(listener=listener.id, **attrs)

        data = utils.get_dict_properties(
            _flatten_listener(obj), self.columns)

        return self.columns, data


class DeleteListener(command.Command):
    _description = _('Delete listener')

    def get_parser(self, prog_name):
        parser = super(DeleteListener, self).get_parser(prog_name)
        parser.add_argument(
            'listener',
            help=_('ID or name of the listener')
        )
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.vlb

        listener = client.find_listener(
            name_or_id=parsed_args.listener,
            ignore_missing=False
        )

        self.app.client_manager.vlb.delete_listener(
            listener=listener.id,
            ignore_missing=False)



#!/usr/bin/env python3
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
#
"""Create a visualization job from attributes."""
import openstack

openstack.enable_logging(True)
conn = openstack.connect(cloud="otc")

attrs = {
    "job_name": "visualization-job",
    "job_desc": "this is a visualization job",
    "train_url": "/obs/name/",
    "job_type": "mindinsight",
    "schedule": [
        {
            "type": "stop",
            "time_unit": "HOURS",
            "duration": 1,
        }
    ],
}

visjob = conn.modelartsv1.create_visualization_job(**attrs)
print(visjob)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from openstack.tests.unit import base

from otcextensions.sdk.nat.v2 import dnat


INSTANCE_ID = '5b95c675-69c2-4656-ba06-58ff72e1d338'
EXAMPLE = {
    'floating_ip_id': 'bdc10a4c-d81a-41ec-adf7-de857f7c812a',
    'status': 'ACTIVE',
    'nat_gateway_id': 'a78fb3eb-1654-4710-8742-3fc49d5f04f8',
    'admin_state_up': True,
    'port_id': '9a469561-daac-4c94-88f5-39366e5ea193',
    'internal_service_port': 993,
    'protocol': 'TCP',
    'project_id': '27e25061336f4af590faeabeb7fcd9a3',
    'created_at': '2017-11-18 07:54:21.665430',
    'id': INSTANCE_ID,
    'floating_ip_address': '5.21.11.226',
    'external_service_port': 242,
    'private_ip': "",
}


class TestDnat(base.TestCase):

    def test_basic(self):
        sot = dnat.Dnat()
        self.assertEqual('dnat_rule', sot.resource_key)
        self.assertEqual('dnat_rules', sot.resources_key)
        path = '/dnat_rules'
        self.assertEqual(path, sot.base_path)
        self.assertTrue(sot.allow_list)
        self.assertTrue(sot.allow_create)
        self.assertTrue(sot.allow_fetch)
        self.assertFalse(sot.allow_commit)
        self.assertTrue(sot.allow_delete)

    def test_make_it(self):
        sot = dnat.Dnat(**EXAMPLE)
        self.assertEqual(EXAMPLE['floating_ip_id'], sot.floating_ip_id)
        self.assertEqual(EXAMPLE['status'], sot.status)
        self.assertEqual(EXAMPLE['nat_gateway_id'], sot.nat_gateway_id)
        self.assertEqual(EXAMPLE['admin_state_up'], sot.admin_state_up)
        self.assertEqual(EXAMPLE['port_id'], sot.port_id)
        self.assertEqual(EXAMPLE['internal_service_port'],
                         sot.internal_service_port)
        self.assertEqual(EXAMPLE['protocol'], sot.protocol)
        self.assertEqual(EXAMPLE['project_id'], sot.project_id)
        self.assertEqual(EXAMPLE['created_at'], sot.created_at)
        self.assertEqual(EXAMPLE['id'], sot.id)
        self.assertEqual(EXAMPLE['floating_ip_address'],
                         sot.floating_ip_address)
        self.assertEqual(EXAMPLE['external_service_port'],
                         sot.external_service_port)
        self.assertEqual(EXAMPLE['private_ip'], sot.private_ip)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from openstack import resource


class Mapping(resource.Resource):
    #: Properties
    #: Disk backup ID
    backup_id = resource.Body('backup_id')
    #: ID of the to which data is restored
    volume_id = resource.Body('volume_id')


class Restore(resource.Resource):
    """CBR Backup Resource"""
    resource_key = 'restore'
    resources_key = ''
    base_path = '/backups/%(backup_id)s/restore'

    # capabilities
    allow_create = True
    allow_list = False
    allow_fetch = False
    allow_delete = False
    allow_commit = False

    _query_mapping = resource.QueryParameters()

    #: Properties
    #: URI backup reference
    backup_id = resource.URI('backup_id')
    #: Restores mapping relationship.
    #: Mandatory for VM restoreation and optional for disk restoration
    mappings = resource.Body('mappings', type=list, list_type=Mapping)
    #: Whether the server is powered on after restoration.
    #: Default: True
    power_on = resource.Body('power_on', type=bool)
    #: ID of the resource to be restored
    resource_id = resource.Body('resource_id')
    #: ID of the target VM to be restored.
    #: Mandatory for VM restoration.
    server_id = resource.Body('server_id')
    #: ID of the target disk to be restored
    #: This parameter is mandatory for disk restoration
    volume_id = resource.Body('volume_id')



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
import mock

from otcextensions.osclient.rds.v3 import flavor
from otcextensions.tests.unit.osclient.rds.v3 import fakes


class TestListDatabaseFlavors(fakes.TestRds):

    objects = fakes.FakeFlavor.create_multiple(3)

    columns = (
        'name', 'instance_mode', 'vcpus', 'ram',
    )

    data = []

    for s in objects:
        data.append(fakes.gen_data(s, columns))

    def setUp(self):
        super(TestListDatabaseFlavors, self).setUp()

        self.cmd = flavor.ListDatabaseFlavors(self.app, None)

        self.client.flavors = mock.Mock(return_value=self.objects)

    def test_list_flavors(self):
        arglist = [
            'MySQL',
            '5.7'
        ]

        verifylist = [
            ('database', 'mysql'),
            ('version', '5.7')
        ]

        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)

        self.client.flavors.assert_called_with(
            datastore_name='mysql',
            version_name='5.7'
        )

        self.assertEqual(self.columns, columns)
        self.assertEqual(self.data, list(data))



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
#
from openstack import resource
from openstack import utils
from otcextensions.sdk.modelartsv2.v2.dataset import LabelSpec


class ResponseSpec(resource.Resource):
    #: Error code.
    error_code = resource.Body("error_code")
    #: Error message.
    error_msg = resource.Body("error_msg")
    #: Check whether the operation is successful.
    success = resource.Body("success", type=bool)


class DatasetLabel(LabelSpec):
    base_path = "/datasets/%(dataset_id)s/data-annotations/labels"

    resources_key = "labels"

    _query_mapping = resource.QueryParameters(
        "version_id",
    )

    # capabilities
    allow_create = True
    allow_list = True

    # Properties
    #: Dataset ID.
    dataset_id = resource.URI("dataset_id")
    #: List of labels to be created.
    labels = resource.Body("labels", type=list, list_type=LabelSpec)
    #: Error code.
    error_code = resource.Body("error_code")
    #: Error message.
    error_msg = resource.Body("error_msg")
    #: Response body for creating a label.
    results = resource.Body("results", type=list, list_type=ResponseSpec)
    #: Check whether the operation is successful.
    success = resource.Body("success", type=bool)

    def update_labels(self, session, labels=[]):
        """Preform actions given the message body."""
        uri = self.base_path % self._uri.attributes
        body = {"labels": labels}
        response = session.put(uri, json=body)
        self._translate_response(response)
        return self

    def delete_labels(self, session, labels=[], delete_policy=None):
        """Preform actions given the message body."""
        params = {}
        if delete_policy:
            params.update(delete_policy=delete_policy)
        uri = utils.urljoin(self.base_path % self._uri.attributes, "delete")
        body = {"labels": labels}
        response = session.post(uri, json=body, params=params)
        self._translate_response(response)
        return self



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from otcextensions.tests.functional.sdk.ctsv3 import TestCtsv3


class TestTraces(TestCtsv3):

    def test_01_list_traces(self):
        attrs = {
            'trace_type': 'system',
            'limit': 2
        }
        traces = list(self.conn.ctsv3.traces(**attrs))
        self.assertGreater(len(traces), 0)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from openstack import proxy
from otcextensions.sdk.deh.v1 import host as _host
from otcextensions.sdk.deh.v1 import server as _server
from otcextensions.sdk.deh.v1 import host_type as _host_type


class Proxy(proxy.Proxy):

    skip_discovery = True

    # ======== hosts ========
    def hosts(self, **query):
        """Retrieve a generator of hosts

        :param dict query: Optional query parameters to be sent to limit the
            resources being returned.

            * `marker`:  pagination marker.
            * `limit`: pagination limit.
            * `id`: Specifies DeH ID.
            * `name`: Specifies the DeH name.
            * `host_type`: Specifes the DeH type.
            * `host_type_name`: Specifes the DeH name of type.
            * `flavor`: Specifies flavor ID.
            * `state`: Specifies the DeH status.
                The value can be available, fault or released.
            * `tenant`: The value can be Tenant ID or all.
                Permits administrator.
            * `availability_host`:  Specifies the AZ to which the DeH belongs.
            * `changes_since`: Filters the response by a date and time
                stamp when the DeH last changed status
                (CCYY-MM-DDThh:mm:ss+hh:mm).
            * `tags`: Specifies tags to use as a filter.

        :returns: A generator of host
            :class:`~otcextensions.sdk.deh.v1.host.Host` instances
        """
        return self._list(_host.Host, **query)

    def create_host(self, **attrs):
        """Create (allocate) a new host from attributes

        :param dict attrs: Keyword arguments which will be used to create
                           a :class:`~otcextensions.sdk.deh.v1.host.Host`,
                           comprised of the properties on the Host class.
        :returns: The results of host creation
        :rtype: :class:`~otcextensions.sdk.deh.v1.host.Host`
        """
        return self._create(_host.Host, prepend_key=False, **attrs)

    def get_host(self, host):
        """Get a host

        :param host: The value can be the ID of a host
             or a :class:`~otcextensions.sdk.deh.v1.host.Host` instance.
        :returns: Host instance
        :rtype: :class:`~otcextensions.sdk.deh.v1.host.Host`
        """
        return self._get(_host.Host, host)

    def delete_host(self, host, ignore_missing=True):
        """Delete (release) a host

        :param host: The value can be the ID of a host
             or a :class:`~otcextensions.sdk.deh.v1.host.Host` instance.
        :param bool ignore_missing: When set to ``False``
            :class:`~openstack.exceptions.ResourceNotFound` will be raised when
            the host does not exist.
            When set to ``True``, no exception will be set when attempting to
            delete a nonexistent host.

        :returns: host been deleted
        :rtype: :class:`~otcextensions.sdk.deh.v1.host.Host`
        """
        return self._delete(_host.Host, host, ignore_missing=ignore_missing)

    def update_host(self, host, **attrs):
        """Update host attributes

        :param host: The id or an instance of
            :class:`~otcextensions.sdk.deh.v1.host.Host`
        :param dict attrs: attributes for update on
            :class:`~otcextensions.sdk.deh.v1.host.Host`

        :rtype: :class:`~otcextensions.sdk.deh.v1.host.Host`
        """
        return self._update(_host.Host, host, **attrs)

    def find_host(self, name_or_id, ignore_missing=True):
        """Find a single host

        :param name_or_id: The name or ID of a host
        :param bool ignore_missing: When set to ``False``
            :class:`~openstack.exceptions.ResourceNotFound` will be raised
            when the host does not exist.
            When set to ``True``, no exception will be set when attempting
            to delete a nonexistent host.

        :returns: ``None``
        """
        return self._find(_host.Host, name_or_id,
                          ignore_missing=ignore_missing)

    # ======== servers ========
    def servers(self, host, **query):
        """Retrieve a generator of servers

        :param host:  The name or ID of a host
        :param dict query: Optional query parameters to be sent to limit the
            resources being returned.
            * `marker`:  pagination marker
            * `limit`: pagination limit

        :returns: A generator of host
            :class:`~otcextensions.sdk.deh.v1.server.Server` instances
        """
        host = self._get_resource(_host.Host, host)
        return self._list(_server.Server, dedicated_host_id=host.id, **query)

    # ======== HostTypes ========
    def host_types(self, az):
        """Retrieve a generator of host types in AZ

        :param az: The availability zone

        :returns: A generator of host
            :class:`~otcextensions.sdk.deh.v1.host_type.HostType` instances
        """
        return self._list(
            _host_type.HostType, availability_zone=az, paginated=False)



#!/usr/bin/env python3
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
import os
import openstack
import uuid
import time
import fixtures

from openstack import resource
from otcextensions.tests.functional import base

_logger = openstack._log.setup_logging('openstack')


class TestShare(base.BaseFunctionalTest):

    TIMEOUT = int(os.environ.get('OS_TEST_TIMEOUT'))
    share = None
    uuid_v4 = uuid.uuid4().hex[:8]
    network_info = None

    def setUp(self):
        test_timeout = 15 * TestShare.TIMEOUT
        try:
            self.useFixture(
                fixtures.EnvironmentVariable(
                    'OS_TEST_TIMEOUT', str(test_timeout)))
        except ValueError:
            pass
        super(TestShare, self).setUp()

    def _create_network(self):
        cidr = '192.168.0.0/16'
        gateway = '192.168.0.1'
        uuid_v4 = uuid.uuid4().hex[:8]
        vpc_name = 'sfsturbo-test-vpc-' + uuid_v4
        subnet_name = 'sfsturbo-test-subnet-' + uuid_v4
        secgroup_name = 'sfsturbo-test-secgroup' + uuid_v4
        secgroup_name_2 = 'sfsturbo-test-secgroup2' + uuid_v4

        if not TestShare.network_info:
            attrs = {
                'name': vpc_name,
                'cidr': cidr
            }
            vpc = self.conn.vpc.create_vpc(**attrs)
            self.assertEqual(vpc_name, vpc.name)
            attrs = {
                'vpc_id': vpc.id,
                'name': subnet_name,
                'cidr': cidr,
                'gateway_ip': gateway,
                'dns_list': [
                    "100.125.4.25",
                    "100.125.129.199",
                ],
            }
            subnet = self.conn.vpc.create_subnet(**attrs)
            resource.wait_for_status(self.conn.vpc, subnet, "ACTIVE", None, 2,
                                     20)

            self.assertEqual(subnet_name, subnet.name)

            sec_group = self.conn.network.create_security_group(
                name=secgroup_name)
            sec_group_2 = self.conn.network.create_security_group(
                name=secgroup_name_2)

            TestShare.network_info = {
                'vpc': vpc,
                'subnet': subnet,
                'secgroup': sec_group,
                'secgroup_2': sec_group_2,
            }

    def _destroy_network(self):
        if TestShare.network_info:
            secgroup = TestShare.network_info['secgroup']
            secgroup_2 = TestShare.network_info['secgroup_2']
            subnet = TestShare.network_info['subnet']
            vpc = TestShare.network_info['vpc']

            time.sleep(100)
            self.conn.vpc.delete_subnet(
                subnet,
                ignore_missing=False
            )
            resource.wait_for_delete(self.conn.vpc, subnet, 2, 120)
            self.conn.vpc.delete_vpc(
                vpc,
                ignore_missing=False
            )

            TestShare.network_info = None

            sot = self.conn.network.delete_security_group(
                secgroup.id,
                ignore_missing=False
            )
            self.assertIsNone(sot)

            sot = self.conn.network.delete_security_group(
                secgroup_2.id,
                ignore_missing=False
            )
            self.assertIsNone(sot)

    def _create_share(self):
        share_name = 'sfsturbo-test-share-' + self.uuid_v4
        TestShare.share = self.conn.sfsturbo.create_share(
            name=share_name,
            vpc_id=TestShare.network_info['vpc'].id,
            subnet_id=TestShare.network_info['subnet'].id,
            security_group_id=TestShare.network_info['secgroup'].id,
            share_proto="NFS",
            share_type="STANDARD",
            size=100,
            availability_zone="eu-de-01"
        )
        self.conn.sfsturbo.wait_for_share(TestShare.share, wait=400)
        self.assertIsNotNone(TestShare.share)

    def test_01_shares(self):
        self._create_network()
        self._create_share()
        shares = list(self.conn.sfsturbo.shares())
        self.assertGreaterEqual(len(shares), 0)

    def test_02_get_share(self):
        share = self.conn.sfsturbo.get_share(TestShare.share.id)
        self.assertEqual(share.id, TestShare.share.id)

    def test_03_extend_capacity(self):
        share = self.conn.sfsturbo.extend_capacity(TestShare.share.id,
                                                   new_size=200)
        self.conn.sfsturbo.wait_for_extend_capacity(share, wait=400)
        share = self.conn.sfsturbo.get_share(share=share.id)
        self.assertEqual(share.avail_capacity, '200.00')

    def test_04_change_security_group(self):
        share = self.conn.sfsturbo.change_security_group(
            TestShare.share.id,
            security_group_id=TestShare.network_info['secgroup_2'].id)
        self.conn.sfsturbo.wait_for_change_security_group(share)
        share = self.conn.sfsturbo.get_share(share=share.id)
        self.assertEqual(share.security_group_id,
                         TestShare.network_info['secgroup_2'].id)

    def test_05_delete_share(self):
        try:
            self.conn.sfsturbo.delete_share(share=TestShare.share)
            self.conn.sfsturbo.wait_for_delete_share(share=TestShare.share)
        except openstack.exceptions.ResourceNotFound:
            self._destroy_network()
            raise
        self._destroy_network()
        try:
            share = self.conn.sfsturbo.get_share(TestShare.share.id)
        except openstack.exceptions.ResourceNotFound:
            share = None
        self.assertIsNone(share)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from openstack.tests.unit import test_proxy_base

from otcextensions.sdk.auto_scaling.v1 import _proxy
from otcextensions.sdk.auto_scaling.v1 import activity as _activity
from otcextensions.sdk.auto_scaling.v1 import config as _config
from otcextensions.sdk.auto_scaling.v1 import group as _group
from otcextensions.sdk.auto_scaling.v1 import instance as _instance
from otcextensions.sdk.auto_scaling.v1 import policy as _policy
from otcextensions.sdk.auto_scaling.v1 import quota as _quota


class TestAutoScalingProxy(test_proxy_base.TestProxyBase):
    def setUp(self):
        super(TestAutoScalingProxy, self).setUp()
        self.proxy = _proxy.Proxy(self.session)


class TestAutoScalingGroups(TestAutoScalingProxy):

    def test_list(self):
        self.verify_list(
            self.proxy.groups, _group.Group,
            method_kwargs={
                'some_arg': 'arg_value',
            },
            expected_kwargs={
                'some_arg': 'arg_value',
            }
        )

    def test_get(self):
        self.verify_get(
            self.proxy.get_group,
            _group.Group,
            expected_kwargs={
            }
        )

    def test_find(self):
        self._verify(
            'openstack.proxy.Proxy._find',
            self.proxy.find_group,
            method_args=["flavor"],
            expected_args=[_group.Group, "flavor"],
            expected_kwargs={
                "ignore_missing": True})

    def test_create(self):
        self.verify_create(
            self.proxy.create_group, _group.Group,
            method_kwargs={
                'instance': 'test',
                'name': 'some_name'
            },
            expected_kwargs={
                'prepend_key': False,
                'instance': 'test',
                'name': 'some_name'
            }
        )

    def test_delete(self):
        self._verify(
            mock_method='otcextensions.sdk.auto_scaling.v1.group.Group.delete',
            test_method=self.proxy.delete_group,
            method_args=['group'],
            expected_args=[self.proxy],
            expected_kwargs={
                'force_delete': False
            }
        )

    def test_delete_with_force_delete(self):
        self._verify(
            mock_method='otcextensions.sdk.auto_scaling.v1.group.Group.delete',
            test_method=self.proxy.delete_group,
            method_args=['group'],
            method_kwargs={'force_delete': True},
            expected_args=[self.proxy],
            expected_kwargs={
                'force_delete': True
            }
        )

    def test_update(self):
        self._verify(
            'openstack.proxy.Proxy._update',
            self.proxy.update_group,
            method_args=['INSTANCE'],
            method_kwargs={'test': 't'},
            expected_args=[_group.Group, 'INSTANCE'],
            expected_kwargs={
                'test': 't',
                'prepend_key': False,
            }
        )


class TestAutoScalingConfigs(TestAutoScalingProxy):

    def test_list(self):
        self.verify_list(
            self.proxy.configs, _config.Config,
            method_kwargs={
                'some_arg': 'arg_value',
            },
            expected_kwargs={
                'some_arg': 'arg_value',
            }
        )

    def test_get(self):
        self.verify_get(
            self.proxy.get_config,
            _config.Config,
            expected_kwargs={
            }
        )

    def test_find(self):
        self._verify(
            'openstack.proxy.Proxy._find',
            self.proxy.find_config,
            method_args=["flavor"],
            expected_args=[_config.Config, "flavor"],
            expected_kwargs={
                "ignore_missing": True})

    def test_create(self):
        self.verify_create(
            self.proxy.create_config, _config.Config,
            method_kwargs={
                'instance_config': {},
                'name': 'some_name'
            },
            expected_kwargs={
                'prepend_key': False,
                'instance_config': {},
                'name': 'some_name'
            }
        )

    def test_delete(self):
        self.verify_delete(
            self.proxy.delete_config,
            _config.Config, True,
            expected_kwargs={
            }
        )


class TestAutoScalingPolicy(TestAutoScalingProxy):

    def test_list(self):
        self.verify_list(
            self.proxy.policies, _policy.Policy,
            method_kwargs={
                'some_arg': 'arg_value',
                'group': 'group_id'
            },
            base_path='/scaling_policy/group_id/list',
            expected_kwargs={
                'some_arg': 'arg_value',
            }
        )

    def test_delete(self):
        self.verify_delete(
            self.proxy.delete_policy,
            _policy.Policy, True,
            expected_kwargs={
            }
        )

    def test_create(self):
        self.verify_create(
            self.proxy.create_policy, _policy.Policy,
            method_kwargs={
                'name': 'some_name'
            },
            expected_kwargs={
                'prepend_key': False,
                'name': 'some_name'
            }
        )

    def test_find(self):
        self._verify(
            'openstack.proxy.Proxy._find',
            self.proxy.find_policy,
            method_args=['pol', 'group'],
            method_kwargs={},
            expected_args=[_policy.Policy, 'pol'],
            expected_kwargs={
                'ignore_missing': True,
                'group_id': 'group'
            }
        )

    def test_update(self):
        self._verify(
            'openstack.proxy.Proxy._update',
            self.proxy.update_policy,
            method_args=['INSTANCE'],
            method_kwargs={'test': 't'},
            expected_args=[_policy.Policy, 'INSTANCE'],
            expected_kwargs={
                'test': 't',
                'prepend_key': False,
            }
        )

    def test_execute(self):
        self._verify(
            'otcextensions.sdk.auto_scaling.v1.policy.Policy._action',
            self.proxy.execute_policy,
            method_args=['INSTANCE'],
            expected_args=[self.proxy, {'action': 'execute'}]
        )

    def test_resume(self):
        self._verify(
            'otcextensions.sdk.auto_scaling.v1.policy.Policy._action',
            self.proxy.resume_policy,
            method_args=['INSTANCE'],
            expected_args=[self.proxy, {'action': 'resume'}]
        )

    def test_pause(self):
        self._verify(
            'otcextensions.sdk.auto_scaling.v1.policy.Policy._action',
            self.proxy.pause_policy,
            method_args=['INSTANCE'],
            expected_args=[self.proxy, {'action': 'pause'}]
        )


class TestAutoScalingActivityLog(TestAutoScalingProxy):

    def test_list(self):
        self.verify_list(
            self.proxy.activities, _activity.Activity,
            method_kwargs={
                'some_arg': 'arg_value',
                'group': 'group_id'
            },
            expected_kwargs={
                'some_arg': 'arg_value',
                'scaling_group_id': 'group_id',
            }
        )


class TestAutoScalingQuota(TestAutoScalingProxy):

    def test_list(self):
        self.verify_list(
            self.proxy.quotas, _quota.Quota,
            expected_kwargs={
                'paginated': False
            }
        )

    def test_list_scaling(self):
        self.verify_list(
            self.proxy.quotas, _quota.ScalingQuota,
            method_args=['INSTANCE'],
            expected_kwargs={
                'paginated': False,
                'scaling_group_id': 'INSTANCE'
            },
            expected_args=[]
        )


class TestAutoScalingInstance(TestAutoScalingProxy):

    def test_list(self):
        self.verify_list(
            self.proxy.instances, _instance.Instance,
            method_args=['group'],
            base_path='/scaling_group_instance/group/list',
            expected_args=[]
        )

    def test_batch_action_remove(self):
        self._verify(
            'otcextensions.sdk.auto_scaling.v1.instance.Instance.batch_action',
            self.proxy.batch_instance_action,
            method_args=['INSTANCE', ['a1', 'a2'], 'REMOVE'],
            expected_args=[self.proxy, ['a1', 'a2'], 'REMOVE', False]
        )

    def test_batch_action_add(self):
        self._verify(
            'otcextensions.sdk.auto_scaling.v1.instance.Instance.batch_action',
            self.proxy.batch_instance_action,
            method_args=['INSTANCE', ['a1', 'a2'], 'ADD'],
            expected_args=[self.proxy, ['a1', 'a2'], 'ADD', False]
        )

    def test_batch_action_protect(self):
        self._verify(
            'otcextensions.sdk.auto_scaling.v1.instance.Instance.batch_action',
            self.proxy.batch_instance_action,
            method_args=['INSTANCE', ['a1', 'a2'], 'PROTECT'],
            expected_args=[self.proxy, ['a1', 'a2'], 'PROTECT', False]
        )

    def test_batch_action_unprotect(self):
        self._verify(
            'otcextensions.sdk.auto_scaling.v1.instance.Instance.batch_action',
            self.proxy.batch_instance_action,
            method_args=['INSTANCE', ['a1', 'a2'], 'UNPROTECT'],
            expected_args=[self.proxy, ['a1', 'a2'], 'UNPROTECT', False]
        )

    def test_delete(self):
        self._verify(
            test_method=self.proxy.remove_instance,
            mock_method='otcextensions.sdk.auto_scaling.v1.'
                        'instance.Instance.remove',
            method_args=['instance'],
            expected_args=[self.proxy],
            expected_kwargs={
                'delete_instance': False
            }
        )

    def test_find(self):
        self.verify_find(
            test_method=self.proxy.find_instance,
            resource_type=_instance.Instance,
            method_kwargs={'group': 'group'},
            expected_kwargs={
                'group_id': 'group'
            }
        )



#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.
#
'''DMS Topic v1 action implementations'''
from osc_lib import utils
from osc_lib.command import command

from otcextensions.common import sdk_utils
from otcextensions.i18n import _


def _get_columns(item):
    column_map = {}
    hidden = ['location']
    return sdk_utils.get_osc_show_columns_for_sdk_resource(item, column_map,
                                                           hidden)


class ListDMSInstanceTopic(command.Lister):
    _description = _('List DMS Instance topics')
    columns = ('ID', 'replication', 'partition', 'retention_time',
               'is_sync_flush', 'is_sync_replication')

    def get_parser(self, prog_name):
        parser = super(ListDMSInstanceTopic, self).get_parser(prog_name)

        parser.add_argument(
            'instance',
            metavar='<instance>',
            help=_('DMS Instance name or ID')
        )
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.dms

        instance_obj = client.find_instance(parsed_args.instance,
                                            ignore_missing=False)
        data = client.topics(instance=instance_obj)

        table = (self.columns,
                 (utils.get_item_properties(
                     s, self.columns,
                 ) for s in data))
        return table


class DeleteDMSInstanceTopic(command.Command):
    _description = _('Delete DMS Instance Topic')

    def get_parser(self, prog_name):
        parser = super(DeleteDMSInstanceTopic, self).get_parser(prog_name)
        parser.add_argument(
            'instance',
            metavar='<instance>',
            help=_('ID or name of the Instance')
        )
        parser.add_argument(
            'topic',
            metavar='<topic>',
            nargs='+',
            help=_('Topic ID')
        )
        return parser

    def take_action(self, parsed_args):

        if parsed_args.instance:
            client = self.app.client_manager.dms
            instance = client.find_instance(
                parsed_args.instance,
                ignore_missing=False)
            client.delete_topic(instance=instance, topics=parsed_args.topic)


class CreateDMSInstanceTopic(command.ShowOne):
    _description = _('Create DMS Instance Topic')

    def get_parser(self, prog_name):
        parser = super(CreateDMSInstanceTopic, self).get_parser(prog_name)
        parser.add_argument(
            'instance',
            metavar='<instance>',
            help=_('Instance ID or Name.')
        )
        parser.add_argument(
            'id',
            metavar='<id>',
            help=_('Name/ID of the topic.')
        )
        parser.add_argument(
            '--partition',
            metavar='<part>',
            type=int,
            choices=range(1, 21),
            help=_(
                'The number of topic partitions, which is used to set the '
                'number of concurrently consumed messages. '
                'Value range: 1–20. Default value: 3.')
        )
        parser.add_argument(
            '--replication',
            metavar='<repl>',
            type=int,
            choices=range(1, 4),
            help=_(
                'The number of replicas, which is configured to ensure data '
                'reliability. Value range: 1–3. Default value: 3.')
        )
        parser.add_argument(
            '--retention-time',
            metavar='<hours>',
            type=int,
            choices=range(1, 169),
            default=72,
            help=_(
                'The retention period of a message. Its default value is '
                '72. Value range: 1–168. Default value: 72. Unit: hour.')
        )
        parser.add_argument(
            '--enable-sync-flush',
            action='store_true',
            help=_(
                'Whether to enable synchronous flushing. '
                'Default value: false. Synchronous flushing compromises '
                'performance.')
        )
        parser.add_argument(
            '--enable-sync-replication',
            action='store_true',
            help=_(
                'Whether to enable synchronous replication. After this '
                'function is enabled, the acks parameter on the producer '
                'client must be set to –1. Otherwise, this parameter does '
                'not take effect.')
        )
        return parser

    def take_action(self, parsed_args):

        attrs = {}

        attrs['id'] = parsed_args.id
        for attr in ['partition', 'replication', 'retention_time']:
            val = getattr(parsed_args, attr)
            if val is not None:
                attrs[attr] = val
        if parsed_args.enable_sync_flush:
            attrs['is_sync_flush'] = True
        if parsed_args.enable_sync_replication:
            attrs['is_sync_replication'] = True

        client = self.app.client_manager.dms

        instance = client.find_instance(parsed_args.instance,
                                        ignore_missing=False)

        obj = client.create_topic(instance=instance, **attrs)

        display_columns, columns = _get_columns(obj)
        data = utils.get_item_properties(obj, columns)

        return (display_columns, data)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
#
import mock

from otcextensions.osclient.vpcep.v1 import quota
from otcextensions.tests.unit.osclient.vpcep.v1 import fakes


class TestListQuota(fakes.TestVpcep):

    objects = fakes.FakeQuota.create_multiple(2)
    column_list_headers = ('Type', 'Quota', 'Used')

    data = []

    for s in objects:
        data.append((s.type, s.quota, s.used))

    def setUp(self):
        super(TestListQuota, self).setUp()

        self.cmd = quota.ListQuota(self.app, None)

        self.client.resource_quota = mock.Mock()
        self.client.api_mock = self.client.resource_quota

    def test_list(self):
        arglist = []

        verifylist = []

        # Verify cm is triggered with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.api_mock.side_effect = [self.objects]

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)

        self.client.api_mock.assert_called_with(None)

        self.assertEqual(self.column_list_headers, columns)
        self.assertEqual(self.data, list(data))

    def test_list_args(self):
        arglist = ['--type', 'endpoint']

        verifylist = [
            ('type', 'endpoint'),
        ]

        # Verify cm is triggered with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.api_mock.side_effect = [self.objects]

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)

        self.client.api_mock.assert_called_with('endpoint')



#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.
#
'''CBR Vault CLI implementation'''
import json
import logging

from osc_lib import utils
from osc_lib.cli import parseractions
from osc_lib.command import command

from otcextensions.i18n import _

LOG = logging.getLogger(__name__)


def _flatten_vault(obj):
    """Flatten the structure of the vault into a single dict
    """
    data = {
        'id': obj.id,
        'name': obj.name,
        'auto_bind': obj.auto_bind,
        'auto_expand': obj.auto_expand,
        'backup_policy_id': obj.backup_policy_id,
        'created_at': obj.created_at,
        'description': obj.description,
        'project_id': obj.project_id,
        'provider_id': obj.provider_id,
        'user_id': obj.user_id,
        'status': obj.billing.status,
        'operation_type': obj.billing.protect_type,
        'object_type': obj.billing.object_type,
        'spec_code': obj.billing.spec_code,
        'size': obj.billing.size,
        'consistent_level': obj.billing.consistent_level,
        'charging_mode': obj.billing.charging_mode,
        'is_auto_pay': obj.billing.is_auto_pay,
        'is_auto_renew': obj.billing.is_auto_renew,
        'bind_rules': obj.bind_rules["tags"],
        'resources': obj.resources,
        'tags': obj.tags,
    }
    return data


def _add_resources_to_vault_obj(obj, data, columns):
    """Add associated resources to column and data tuples
    """
    i = 0
    for s in obj.resources:
        if obj.resources[i].id:
            name = 'resource_id_' + str(i + 1)
            data += (obj.resources[i].id,)
            columns = columns + (name,)

            name = 'resource_type_' + str(i + 1)
            data += (obj.resources[i].type,)
            columns = columns + (name,)

            i += 1
    return data, columns


def _add_tags_to_vault_obj(obj, data, columns):
    data += ('\n'.join((f'value={tag["value"]}, key={tag["key"]}'
                        for tag in obj.tags)),)
    columns = columns + ('tags',)
    return data, columns


def _add_bind_rules_to_vault_obj(obj, data, columns):
    """Add associated bind rules to column and data tuples
    """
    data += ('\n'.join((f'value={tag["value"]}, key={tag["key"]}'
                        for tag in obj.bind_rules['tags'])),)
    columns = columns + ('bind_rules',)
    return data, columns


def _add_associated_policy_to_vault_obj(data, columns):
    """Add associated resources to column and data tuples
    """
    return_data = ()
    for k, v in data['associate_policy'].items():
        return_data += (v,)
        columns = columns + (k,)
    return return_data, columns


def _add_associated_resources_to_vault_obj(data, columns):
    """Add associated resources to column and data tuples
    """
    return_data = ()
    i = 0
    for s in data['add_resource_ids']:
        name = 'resource_' + str(i + 1)
        return_data += (s,)
        columns = columns + (name,)
        i += 1
    return return_data, columns


def _normalize_tags(tags):
    result = []
    for tag in tags:
        try:
            tag = tag.split('=')
            result.append({
                'key': tag[0],
                'value': tag[1]
            })
        except IndexError:
            result.append({'key': tag[0], 'value': ''})
    return result


class ListVaults(command.Lister):
    _description = _('List CBR Vaults')
    columns = ('ID', 'name', 'backup_policy_id', 'description', 'created_at')

    def get_parser(self, prog_name):
        parser = super(ListVaults, self).get_parser(prog_name)
        parser.add_argument(
            '--id',
            metavar='<id>',
            help=_('Vault ID.')
        )
        parser.add_argument(
            '--name',
            metavar='<name>',
            help=_('Vault name')
        )
        parser.add_argument(
            '--cloud-type',
            metavar='<cloud_type>',
            help=_('Cloud type, which is public.')
        )
        parser.add_argument(
            '--limit',
            type=int,
            metavar='<limit>',
            help=_('Limit.')
        )
        parser.add_argument(
            '--object-type',
            metavar='<object_type>',
            help=_('Object type, which can be server or disk.')
        )
        parser.add_argument(
            '--offset',
            type=int,
            metavar='<offset>',
            help=_('Offset value. The value must be a positive integer.')
        )
        parser.add_argument(
            '--policy-id',
            metavar='<policy_id>',
            help=_('Policy ID.')
        )
        parser.add_argument(
            '--protect-type',
            metavar='<protect_type>',
            help=_('Protection type, which is backup.')
        )
        parser.add_argument(
            '--status',
            metavar='<status>',
            help=_('Status.')
        )
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.cbr
        args = {}
        if parsed_args.id:
            args['id'] = parsed_args.id
        if parsed_args.name:
            args['name'] = parsed_args.name
        if parsed_args.cloud_type:
            args['cloud_type'] = parsed_args.cloud_type
        if parsed_args.limit:
            args['limit'] = parsed_args.limit
        if parsed_args.object_type:
            args['object_type'] = parsed_args.object_type
        if parsed_args.offset:
            args['offset'] = parsed_args.offset
        if parsed_args.policy_id:
            args['policy_id'] = parsed_args.policy_id
        if parsed_args.protect_type:
            args['protect_type'] = parsed_args.protect_type
        if parsed_args.status:
            args['status'] = parsed_args.status

        data = client.vaults(**args)
        table = (self.columns,
                 (utils.get_dict_properties(
                     _flatten_vault(s), self.columns,
                 ) for s in data))
        return table


class ShowVault(command.ShowOne):
    _description = _('Show single Vault details')
    columns = (
        'ID',
        'name',
        'auto_bind',
        'auto_expand',
        'backup_policy_id',
        'created_at',
        'description',
        'project_id',
        'provider_id',
        'user_id',
        'status',
        'operation_type',
        'object_type',
        'spec_code',
        'size',
        'consistent_level',
        'charging_mode',
        'is_auto_pay',
        'is_auto_renew',
    )

    def get_parser(self, prog_name):
        parser = super(ShowVault, self).get_parser(prog_name)
        parser.add_argument(
            'vault',
            metavar='<vault>',
            help=_('ID or name of the CBR vault.')
        )
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.cbr

        obj = client.find_vault(
            name_or_id=parsed_args.vault,
            ignore_missing=False
        )

        data = utils.get_dict_properties(
            _flatten_vault(obj), self.columns)

        if obj.resources:
            data, self.columns = _add_resources_to_vault_obj(
                obj, data, self.columns)
        if obj.tags:
            data, self.columns = _add_tags_to_vault_obj(
                obj, data, self.columns)
        if obj.bind_rules.get('tags'):
            if obj.bind_rules['tags']:
                data, self.columns = _add_bind_rules_to_vault_obj(
                    obj, data, self.columns)
        return self.columns, data


class CreateVault(command.ShowOne):
    _description = _('Create CBR Vault')
    columns = (
        'ID',
        'name',
        'auto_bind',
        'auto_expand',
        'backup_policy_id',
        'created_at',
        'description',
        'project_id',
        'provider_id',
        'user_id',
        'status',
        'operation_type',
        'object_type',
        'spec_code',
        'size',
        'consistent_level',
        'charging_mode',
        'is_auto_pay',
        'is_auto_renew',
    )

    def get_parser(self, prog_name):
        parser = super(CreateVault, self).get_parser(prog_name)
        parser.add_argument(
            'name',
            metavar='<name>',
            help=_('Name of the CBR Vault.')
        )
        parser.add_argument(
            '--backup-policy',
            metavar='<backup_policy>',
            help=_('Name or id of the CBR Policy.')
        )
        parser.add_argument(
            '--description',
            metavar='<description>',
            help=_('User-defined CBR Vault description.')
        )
        parser.add_argument(
            '--enterprise-project-id',
            metavar='<enterprise_project_id>',
            help=_('Enterprise project ID.')
        )
        parser.add_argument(
            '--auto-bind',
            metavar='<auto_bind>',
            type=bool,
            help=_('Whether automatic association is supported.')
        )
        parser.add_argument(
            '--bind-rule',
            metavar='<bind_rule>',
            action='append',
            help=_('Filters automatically associated resources by tag '
                   'in KEY=VALUE format. '
                   'Repeat for multiple values.')
        )
        parser.add_argument(
            '--consistent-level',
            metavar='<consistent_level>',
            required=True,
            default='crash_consistent',
            choices=['crash_consistent', 'app_consistent'],
            help=_('Backup specifications, which can be \n'
                   'crash_consistent (crash consistent backup) or \n'
                   'app_consistent (application consistency backup) '
                   'Backup specifications.')
        )
        parser.add_argument(
            '--object-type',
            metavar='<object_type>',
            required=True,
            choices=['server', 'disk', 'turbo'],
            help=_('Object type.')
        )
        parser.add_argument(
            '--size',
            metavar='<size>',
            type=int,
            required=True,
            help=_('Capacity, in GB.\n'
                   'Ranges from 1 to 10485760.')
        )
        parser.add_argument(
            '--is-auto-renew',
            action='store_false',
            help=_('Whether to automatically renew the subscription '
                   'after expiration. By default, it is not renewed.')
        )
        parser.add_argument(
            '--is-auto-pay',
            action='store_false',
            help=_('Whether the fee is automatically deducted from the '
                   'customer account balance after an order is submitted. '
                   'The non-automatic payment mode is used by default.')
        )
        parser.add_argument(
            '--console-url',
            metavar='<console_url>',
            help=_('Redirection URL.')
        )
        parser.add_argument(
            '--resource',
            metavar='name=<name>,value=<value>,type=<type>',
            action=parseractions.MultiKeyValueAction,
            dest='resource',
            required_keys=['id', 'type'],
            optional_keys=['name'],
            help=_('Associated resource in "id=resource_id,'
                   'type=resource_type,name=resource_name" format.'
                   'Repeat for multiple values.')
        )
        parser.add_argument(
            '--tag',
            metavar='<tag>',
            action='append',
            help=_('Tag to assign to the server in KEY=VALUE format. '
                   'Repeat for multiple values.')
        )
        return parser

    def take_action(self, parsed_args):
        attrs = {
            'resources': [],
            'bind_rules': {
                'tags': []
            },
            'billing': {
                'cloud_type': 'public',
                'protect_type': 'backup',
                'charging_mode': 'post_paid'
            }
        }

        # mandatory
        attrs['name'] = parsed_args.name

        attrs['billing']['consistent_level'] = parsed_args.consistent_level
        attrs['billing']['object_type'] = parsed_args.object_type
        attrs['billing']['size'] = parsed_args.size

        if parsed_args.resource:
            attrs['resources'] = parsed_args.resource

        # optional
        if parsed_args.is_auto_renew:
            attrs['billing']['is_auto_renew'] = parsed_args.is_auto_renew
        if parsed_args.is_auto_pay:
            attrs['billing']['is_auto_pay'] = parsed_args.is_auto_pay
        if parsed_args.console_url:
            attrs['billing']['console_url'] = parsed_args.console_url
        if parsed_args.backup_policy:
            attrs['backup_policy_id'] = parsed_args.backup_policy
        if parsed_args.bind_rule:
            attrs['bind_rules'] = {'tags': []}
            attrs['bind_rules']['tags'] = _normalize_tags(
                parsed_args.bind_rule)
        if parsed_args.description:
            attrs['description'] = parsed_args.description
        if parsed_args.enterprise_project_id:
            attrs['enterprise_project_id'] = parsed_args.enterprise_project_id
        if parsed_args.auto_bind:
            attrs['auto_bind'] = parsed_args.auto_bind
        if parsed_args.tag:
            attrs['tags'] = _normalize_tags(parsed_args.tag)

        client = self.app.client_manager.cbr
        obj = client.create_vault(**attrs)

        data = utils.get_dict_properties(
            _flatten_vault(obj), self.columns)

        if obj.resources:
            data, self.columns = _add_resources_to_vault_obj(
                obj, data, self.columns)
        if obj.bind_rules.get('tags'):
            if obj.bind_rules['tags']:
                data, self.columns = _add_bind_rules_to_vault_obj(
                    obj, data, self.columns)
        if obj.tags:
            data, self.columns = _add_tags_to_vault_obj(
                obj, data, self.columns)

        return self.columns, data


class UpdateVault(command.ShowOne):
    _description = _('Update CBR Vault')
    columns = (
        'ID',
        'name',
        'auto_bind',
        'auto_expand',
        'backup_policy_id',
        'created_at',
        'description',
        'project_id',
        'provider_id',
        'user_id',
        'status',
        'operation_type',
        'object_type',
        'spec_code',
        'size',
        'consistent_level',
        'charging_mode',
        'is_auto_pay',
        'is_auto_renew',
    )

    def get_parser(self, prog_name):
        parser = super(UpdateVault, self).get_parser(prog_name)
        parser.add_argument(
            'vault',
            metavar='<vault>',
            help=_('ID or name of the CBR Vault.')
        )
        parser.add_argument(
            '--name',
            metavar='<name>',
            help=_('Name of the CBR Vault.')
        )
        parser.add_argument(
            '--auto-bind',
            metavar='<auto_bind>',
            type=bool,
            help=_('Whether automatic association is supported.')
        )
        parser.add_argument(
            '--bind-rule',
            metavar='<bind_rule>',
            action='append',
            help=_('Filters automatically associated resources by tag '
                   'in KEY=VALUE format. '
                   'Repeat for multiple values.')
        )
        parser.add_argument(
            '--auto-expand',
            metavar='<auto_expand>',
            type=bool,
            help=_('Whether to enable auto capacity expansion for the vault.')
        )
        parser.add_argument(
            '--smn-notify',
            metavar='<smn_notify>',
            type=bool,
            help=_('Exception notification function.')
        )
        parser.add_argument(
            '--size',
            metavar='<size>',
            type=int,
            help=_('Capacity, in GB.\n'
                   'Ranges from 1 to 10485760.')
        )
        parser.add_argument(
            '--threshold',
            metavar='<threshold>',
            type=int,
            help=_('Vault capacity threshold. If the vault capacity usage '
                   'exceeds this threshold and smn_notify is true, '
                   'an exception notification is sent.\n'
                   'Ranges from 1 to 100.')
        )

        return parser

    def take_action(self, parsed_args):
        attrs = {
            'billing': {},
        }

        if parsed_args.name:
            attrs['name'] = parsed_args.name
        if parsed_args.auto_bind:
            attrs['auto_bind'] = parsed_args.auto_bind
        if parsed_args.bind_rule:
            attrs['bind_rules'] = {'tags': []}
            attrs['bind_rules']['tags'] = _normalize_tags(
                parsed_args.bind_rule
            )
        if parsed_args.auto_expand:
            attrs['auto_expand'] = parsed_args.auto_expand
        if parsed_args.smn_notify:
            attrs['smn_notify'] = parsed_args.smn_notify
        if parsed_args.size:
            attrs['billing']['size'] = parsed_args.size
        if parsed_args.threshold:
            attrs['threshold'] = parsed_args.threshold

        client = self.app.client_manager.cbr
        vault = client.find_vault(
            name_or_id=parsed_args.vault,
            ignore_missing=False
        )

        if attrs:
            obj = client.update_vault(vault=vault.id, **attrs)
        else:
            obj = vault

        data = utils.get_dict_properties(
            _flatten_vault(obj), self.columns)

        if obj.resources:
            data, self.columns = _add_resources_to_vault_obj(
                obj, data, self.columns)
        if obj.bind_rules.get('tags'):
            if obj.bind_rules['tags']:
                data, self.columns = _add_bind_rules_to_vault_obj(
                    obj, data, self.columns)
        if obj.tags:
            data, self.columns = _add_tags_to_vault_obj(
                obj, data, self.columns)

        return self.columns, data


class DeleteVault(command.Command):
    _description = _('Delete CBR Vault')

    def get_parser(self, prog_name):
        parser = super(DeleteVault, self).get_parser(prog_name)
        parser.add_argument(
            'vault',
            metavar='<vault>',
            help=_('ID or name of the CBR Vault.')
        )
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.cbr

        vault = client.find_vault(
            name_or_id=parsed_args.vault,
            ignore_missing=False
        )

        self.app.client_manager.cbr.delete_vault(
            vault=vault.id,
            ignore_missing=False)


class AssociateVaultResource(command.ShowOne):
    _description = _('Associate resource to the CBR Vault')

    columns = ()

    def get_parser(self, prog_name):
        parser = super(AssociateVaultResource, self).get_parser(prog_name)
        parser.add_argument(
            'vault',
            metavar='<vault>',
            help=_('ID or name of the CBR Vault.')
        )
        parser.add_argument(
            '--resource',
            metavar='name=<name>,value=<value>,type=<type>',
            action=parseractions.MultiKeyValueAction,
            dest='resource',
            required_keys=['id', 'type'],
            optional_keys=['name'],
            help=_('Associated resource in "id=resource_id,'
                   'type=resource_type,name=resource_name" format.'
                   'Repeat for multiple values.')
        )
        return parser

    def take_action(self, parsed_args):

        resources = []

        client = self.app.client_manager.cbr
        vault = client.find_vault(
            name_or_id=parsed_args.vault,
            ignore_missing=False
        )

        resources = parsed_args.resource

        if resources:
            obj = client.associate_resources(
                vault=vault.id,
                resources=resources
            )

            data = json.loads(obj._content.decode('utf-8'))

            if data['add_resource_ids']:
                data, self.columns = _add_associated_resources_to_vault_obj(
                    data, self.columns)

            return self.columns, data


class DissociateVaultResource(command.Command):
    _description = _('Dissociate resource from the CBR Vault')

    def get_parser(self, prog_name):
        parser = super(DissociateVaultResource, self).get_parser(prog_name)
        parser.add_argument(
            'vault',
            metavar='<vault>',
            help=_('ID or name of the CBR Vault.')
        )
        parser.add_argument(
            '--resource',
            metavar='<resource>',
            action='append',
            help=_('Removed resource IDs.'
                   'Repeat for multiple values.')
        )
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.cbr

        vault = client.find_vault(
            name_or_id=parsed_args.vault,
            ignore_missing=False
        )

        if parsed_args.resource:
            client.dissociate_resources(
                vault=vault.id,
                resources=parsed_args.resource
            )


class BindVaultPolicy(command.ShowOne):
    _description = _('Bind policy to the CBR Vault')

    columns = ()

    def get_parser(self, prog_name):
        parser = super(BindVaultPolicy, self).get_parser(prog_name)
        parser.add_argument(
            'vault',
            metavar='<vault>',
            help=_('ID or name of the CBR Vault.')
        )
        parser.add_argument(
            'policy',
            metavar='<policy>',
            help=_('ID or name of the CBR Vault.')
        )
        return parser

    def take_action(self, parsed_args):

        client = self.app.client_manager.cbr
        vault = client.find_vault(
            name_or_id=parsed_args.vault,
            ignore_missing=False
        )

        if parsed_args.policy:
            obj = client.bind_policy(
                vault=vault.id,
                policy=parsed_args.policy
            )

            data = json.loads(obj._content.decode('utf-8'))
            if data['associate_policy']:
                data, self.columns = _add_associated_policy_to_vault_obj(
                    data, self.columns)

            return self.columns, data


class UnbindVaultPolicy(command.Command):
    _description = _('Unbind policy from the CBR Vault')

    def get_parser(self, prog_name):
        parser = super(UnbindVaultPolicy, self).get_parser(prog_name)
        parser.add_argument(
            'vault',
            metavar='<vault>',
            help=_('ID or name of the CBR Vault.')
        )
        parser.add_argument(
            'policy',
            metavar='<policy>',
            help=_('ID or name of the CBR Vault.')
        )
        return parser

    def take_action(self, parsed_args):

        client = self.app.client_manager.cbr
        vault = client.find_vault(
            name_or_id=parsed_args.vault,
            ignore_missing=False
        )

        client.unbind_policy(
            vault=vault.id,
            policy=parsed_args.policy
        )



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
#
import mock
from openstackclient.tests.unit import utils as tests_utils
from osc_lib import exceptions

from otcextensions.common import cli_utils
from otcextensions.osclient.modelartsv1.v1 import training_job_version
from otcextensions.tests.unit.osclient.modelartsv1.v1 import fakes

_COLUMNS = (
    "attributes",
    "billing",
    "core",
    "cpu",
    "created_at",
    "data_url",
    "dataset_id",
    "dataset_name",
    "dataset_version_id",
    "dataset_version_name",
    "description",
    "duration",
    "engine_id",
    "engine_name",
    "engine_type",
    "engine_version",
    "flavor_code",
    "flavor_info",
    "flavor_type",
    "gpu_memory_unit",
    "gpu_num",
    "gpu_type",
    "id",
    "is_free",
    "is_success",
    "job_description",
    "job_name",
    "job_type",
    "max_num",
    "memory_unit",
    "model_id",
    "model_metric_list",
    "name",
    "nas_type",
    "no_resource",
    "parameter",
    "pod_version",
    "pool_id",
    "pool_name",
    "pool_type",
    "resource_id",
    "spec_code",
    "spec_id",
    "started_at",
    "status",
    "system_metric_list",
    "train_url",
    "version_id",
    "version_name",
    "worker_server_num",
)


class TestListTrainingJobVersions(fakes.TestModelartsv1):
    job_id = "123"

    objects = fakes.FakeTrainingJobVersion.create_multiple(3)

    column_list_headers = (
        "Version Id",
        "Version Name",
        "Status",
        "Created At",
        "Started At",
        "Duration",
    )

    data = []

    for s in objects:
        data.append(
            (
                s.version_id,
                s.version_name,
                training_job_version.JobStatus(s.status),
                cli_utils.UnixTimestampFormatter(s.created_at),
                cli_utils.UnixTimestampFormatter(s.started_at),
                s.duration,
            )
        )

    def setUp(self):
        super(TestListTrainingJobVersions, self).setUp()

        self.cmd = training_job_version.ListTrainingJobVersions(self.app, None)

        self.client.training_job_versionss = mock.Mock()
        self.client.api_mock = self.client.training_job_versions

    def test_list(self):
        arglist = [self.job_id]

        verifylist = [
            ("jobId", self.job_id),
        ]

        # Verify cm is triggered with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.api_mock.side_effect = [self.objects]

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)

        self.client.api_mock.assert_called_with(self.job_id)

        self.assertEqual(self.column_list_headers, columns)
        self.assertEqual(self.data, list(data))

    def test_list_args(self):
        arglist = [
            self.job_id,
            "--limit",
            "1",
            "--offset",
            "2",
        ]

        verifylist = [
            ("jobId", self.job_id),
            ("limit", 1),
            ("offset", 2),
        ]

        # Verify cm is triggered with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.api_mock.side_effect = [self.objects]

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)

        self.client.api_mock.assert_called_with(self.job_id, limit=1, offset=2)


class TestCreateTrainingJobVersion(fakes.TestModelartsv1):
    job_id = "123"

    _data = fakes.FakeTrainingJobVersion.create_one()
    columns = _COLUMNS

    data = fakes.gen_data(_data, columns, training_job_version._formatters)

    def setUp(self):
        super(TestCreateTrainingJobVersion, self).setUp()

        self.cmd = training_job_version.CreateTrainingJobVersion(
            self.app, None
        )

        self.client.create_training_job_version = mock.Mock(
            return_value=self._data
        )

    def test_create(self):
        arglist = [
            self.job_id,
            "--description",
            "1",
            "--pre-version-id",
            "2",
            "--worker-server-num",
            "3",
            "--app-url",
            "4",
            "--boot-file-url",
            "5",
            "--log-url",
            "6",
            "--data-url",
            "7",
            "--dataset-id",
            "8",
            "--dataset-version",
            "9",
            "--spec-id",
            "11",
            "--engine-id",
            "12",
            "--model-id",
            "13",
            "--train-url",
            "14",
            "--parameter",
            "label=k1,value=v1",
            "--parameter",
            "label=k2,value=v2",
        ]
        verifylist = [
            ("jobId", self.job_id),
            ("pre_version_id", 2),
            ("job_desc", "1"),
            ("worker_server_num", 3),
            ("app_url", "4"),
            ("boot_file_url", "5"),
            ("log_url", "6"),
            ("data_url", "7"),
            ("dataset_id", "8"),
            ("dataset_version_id", "9"),
            ("spec_id", 11),
            ("engine_id", 12),
            ("model_id", 13),
            ("train_url", "14"),
            (
                "parameter",
                [
                    {"label": "k1", "value": "v1"},
                    {"label": "k2", "value": "v2"},
                ],
            ),
        ]
        # Verify cm is triggereg with default parameters
        self.check_parser(self.cmd, arglist, verifylist)
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)
        attrs = {
            "job_desc": "1",
            "config": {
                "worker_server_num": 3,
                "pre_version_id": 2,
                "app_url": "4",
                "boot_file_url": "5",
                "parameter": [
                    {"label": "k1", "value": "v1"},
                    {"label": "k2", "value": "v2"},
                ],
                "data_url": "7",
                "dataset_id": "8",
                "dataset_version_id": "9",
                "spec_id": 11,
                "engine_id": 12,
                "model_id": 13,
                "train_url": "14",
                "log_url": "6",
            },
        }
        self.client.create_training_job_version.assert_called_with(
            self.job_id, **attrs
        )
        self.assertEqual(self.columns, columns)
        self.assertEqual(self.data, data)


class TestShowTrainingJobVersion(fakes.TestModelartsv1):
    job_id = "123"
    version_id = "2"

    _data = fakes.FakeTrainingJobVersion.create_one()
    columns = _COLUMNS
    data = fakes.gen_data(_data, columns, training_job_version._formatters)

    def setUp(self):
        super(TestShowTrainingJobVersion, self).setUp()

        self.cmd = training_job_version.ShowTrainingJobVersion(self.app, None)

        self.client.get_training_job_version = mock.Mock(
            return_value=self._data
        )

    def test_show_no_options(self):
        arglist = []
        verifylist = []

        # Testing that a call without the required argument will fail and
        # throw a "ParserExecption"
        self.assertRaises(
            tests_utils.ParserException,
            self.check_parser,
            self.cmd,
            arglist,
            verifylist,
        )

    def test_show(self):
        arglist = [self.job_id, self.version_id]

        verifylist = [
            ("jobId", self.job_id),
            ("versionId", self.version_id),
        ]

        # Verify cm is triggered with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)
        self.client.get_training_job_version.assert_called_with(
            self.job_id, self.version_id
        )

        self.assertEqual(self.columns, columns)
        self.assertEqual(self.data, data)

    def test_show_non_existent(self):
        arglist = [self.job_id, "non-existing-version-id"]

        verifylist = [
            ("jobId", self.job_id),
            ("versionId", "non-existing-version-id"),
        ]

        # Verify cm is triggered with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        self.client.get_training_job_version = mock.Mock(
            side_effect=exceptions.CommandError("Resource Not Found")
        )

        # Trigger the action
        try:
            self.cmd.take_action(parsed_args)
        except Exception as e:
            self.assertEqual("Resource Not Found", str(e))
        self.client.get_training_job_version.assert_called_with(
            self.job_id, "non-existing-version-id"
        )


class TestStopTrainingJobVersion(fakes.TestModelartsv1):

    job_id = "123"
    version_id = "2"

    _data = fakes.FakeTrainingJobVersion.create_one()
    columns = _COLUMNS
    data = fakes.gen_data(_data, columns, training_job_version._formatters)

    def setUp(self):
        super(TestStopTrainingJobVersion, self).setUp()

        self.cmd = training_job_version.StopTrainingJobVersion(self.app, None)

        self.client.stop_training_job_version = mock.Mock(
            return_value=self._data
        )

    def test_start(self):
        arglist = [
            self.job_id,
            self.version_id,
        ]

        verifylist = [
            ("jobId", self.job_id),
            ("versionId", self.version_id),
        ]

        # Verify cm is triggered with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Trigger the action
        self.cmd.take_action(parsed_args)
        self.client.stop_training_job_version.assert_called_with(
            self.job_id, self.version_id
        )


class TestDeleteTrainingJobVersion(fakes.TestModelartsv1):

    job_id = "123"
    version_id = "2"

    def setUp(self):
        super(TestDeleteTrainingJobVersion, self).setUp()

        self.client.delete_training_job_version = mock.Mock(return_value=None)

        # Get the command object to test
        self.cmd = training_job_version.DeleteTrainingJobVersion(
            self.app, None
        )

    def test_delete(self):
        arglist = [self.job_id, self.version_id]

        verifylist = [
            ("jobId", self.job_id),
            ("versionId", self.version_id),
        ]

        # Verify cm is triggered with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Trigger the action
        result = self.cmd.take_action(parsed_args)

        self.client.delete_training_job_version.assert_called_with(
            self.job_id, self.version_id
        )
        self.assertIsNone(result)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
import os
import uuid

from otcextensions.tests.functional.sdk.swr import TestSwr


class TestRepository(TestSwr):

    def setUp(self):
        super(TestRepository, self).setUp()

        self.org_name = "sdk-swr-org-" + uuid.uuid4().hex
        self.repo_name = "sdk-swr-repo-" + uuid.uuid4().hex
        self.org = self.client.create_organization(
            namespace=self.org_name
        )
        self.repo = self.client.create_repository(
            namespace=self.org_name,
            repository=self.repo_name,
            category='linux',
            description='this is a acc test repository',
            is_public=True,
        )
        if os.getenv("OS_SWR_PERMISSIONS_RUN"):
            self.permission = [
                {
                    "user_id": "5a23ecb3999b458d92d51d524bb7fb4b",
                    "user_name": "pgubina",
                    "user_auth": 1
                }
            ]
            self.repo_perm = self.client.create_repository_permissions(
                namespace=self.org_name,
                repository=self.repo_name,
                permissions=self.permission
            )

    def tearDown(self):
        super(TestRepository, self).tearDown()
        if os.getenv("OS_SWR_PERMISSIONS_RUN"):
            self.client.delete_repository_permissions(
                namespace=self.org.namespace,
                repository=self.repo.repository,
                user_ids=[self.repo_perm.permissions[0].user_id]
            )
        self.conn.swr.delete_repository(
            self.org_name,
            self.repo_name
        )
        self.conn.swr.delete_organization(self.org_name)

    def test_get_repository(self):
        repo = self.client.get_repository(
            namespace=self.org.namespace,
            repository=self.repo_name
        )
        self.assertEqual(self.repo_name, repo.name)
        self.assertEqual(True, repo.is_public)
        self.assertEqual(self.org_name, repo.namespace)

    def test_list_repositories(self):
        query = {
            'namespace': self.org.namespace
        }
        repos = list(self.client.repositories(**query))
        self.assertGreaterEqual(len(repos), 0)

    def test_update_repository(self):
        desc = 'this is a acc test repository updated'
        updated = self.client.update_repository(
            namespace=self.org_name,
            repository=self.repo_name,
            category='windows',
            description=desc,
        )
        self.assertEqual(desc, updated.description)

    def test_repository_permissions(self):
        o = list(self.client.repository_permissions(
            self.org.namespace,
            self.repo.repository
        ))
        self.assertEqual(self.org.namespace, o[0].namespace)

    def test_update_repository_permissions(self):
        if os.getenv("OS_SWR_PERMISSIONS_RUN"):
            o = self.client.update_repository_permissions(
                namespace=self.org_name,
                repository=self.repo_name,
                permissions=[
                    {
                        "user_id": "5a23ecb3999b458d92d51d524bb7fb4b",
                        "user_name": "pgubina",
                        "user_auth": 3
                    }
                ]
            )
            self.assertEqual(3, o.permissions[0].user_auth)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from openstack import proxy

from otcextensions.common.utils import extract_url_parts
from otcextensions.sdk.anti_ddos.v1 import alert_config as _alert
from otcextensions.sdk.anti_ddos.v1 import config as _config
from otcextensions.sdk.anti_ddos.v1 import floating_ip as _floating_ip
from otcextensions.sdk.anti_ddos.v1 import status as _status


class Proxy(proxy.Proxy):

    skip_discovery = True

    def _extract_name(self, url, service_type=None, project_id=None):
        return extract_url_parts(url, project_id)

    # ======== Alert Config ========
    def get_alert_config(self, **kwargs):
        """Get Alarm configuration

        :returns: one object of class
            :class:`~otcextensions.sdk.anti_ddos.v1.alert_config.AlertConfig`
        """
        return self._get(_alert.AlertConfig, None, requires_id=False)

    # ======== Configurations ========
    def configs(self, **kwargs):
        """Get Alarm configuration

        :returns: one object of class
            :class:`~otcextensions.sdk.anti_ddos.v1.config.Config`
        """
        return self._list(_config.Config, paginated=False, **kwargs)

    # ======== FloatIP DDoS ========
    def floating_ips(self, **query):
        """Get the list of anti-ddos Floating IPs

        :param kwargs query: Optional query parameters to be sent to limit
            the resources being returned.

        :returns: A generator of FloatingIP object
        :rtype: class:`~otcextensions.sdk.anti_ddos.v1.floating_ip.FloatingIP`
        """

        return self._list(_floating_ip.FloatingIP, paginated=False, **query)

    def protect_floating_ip(self, floating_ip_id, **kwargs):
        """Enable anti-ddos on the given Floating IP

        :param floating_ip_id: The Floating IP id or an instance of
            :class:`~otcextensions.sdk.anti_ddos.v1.floating_ip.FloatingIP`
        :param dict kwargs: Keyword arguments which will be used to create a
            :class:`~otcextensions.sdk.anti_ddos.v1.floating_ip.FloatingIP`

        :rtype: :class:`~otcextensions.sdk.anti_ddos.v1.floating_ip.FloatingIP`
        """
        return self._create(_floating_ip.FloatingIP,
                            floating_ip_id=floating_ip_id, **kwargs)

    def unprotect_floating_ip(self, floating_ip, ignore_missing=True):
        """Diable anti-ddos on the given Floating IP

        :param floating_ip_id: The EIP id or an instance of
            :class:`~otcextensions.sdk.anti_ddos.v1.floating_ip.FloatingIP`
        :param bool ignore_missing: When set to ``False``
            :class:`~otcextensions.sdk.exceptions.ResourceNotFound` will be
            raised when the floating IP does not exist.

        :rtype: :class:`~otcextensions.sdk.anti_ddos.v1.floating_ip.FloatingIP`
        """
        return self._delete(_floating_ip.FloatingIP,
                            floating_ip,
                            ignore_missing=ignore_missing)

    def get_floating_ip_policies(self, floating_ip):
        """Get detail about an Floating IP policy

        :param floating_ip: The Floating IP id or an instance of
            :class:`~otcextensions.sdk.anti_ddos.v1.floating_ip.FloatingIP`

        :rtype: :class:`~otcextensions.sdk.anti_ddos.v1.floating_ip.FloatingIP`
        """
        return self._get(_floating_ip.FloatingIP, floating_ip)

    def update_floating_ip_policies(self, floating_ip, **attrs):
        """Update Floating IP policy

        :param floating_ip: The Floating IP id or an instance of
            :class:`~otcextensions.sdk.anti_ddos.v1.floating_ip.FloatingIP`
        :param dict attrs: attributes for update on
            :class:`~otcextensions.sdk.anti_ddos.v1.floating_ip.FloatingIP`

        :rtype: :class:`~otcextensions.sdk.anti_ddos.v1.floating_ip.FloatingIP`
        """
        return self._update(_floating_ip.FloatingIP, floating_ip, **attrs)

    def get_floating_ip_status(self, floating_ip_id):
        """Get specific floating ip status by floating ip id.

        :param floating_ip_id: The floating ip id
        :returns: The status of floating ip
        :rtype:
            :class:`~otcextensions.sdk.anti_ddos.v1.status.FloatingIPStatus`
        """
        return self._get(
            _status.FloatingIPStatus,
            floating_ip_id,
            requires_id=False,
        )

    def floating_ip_events(self, floating_ip_id, **query):
        """List specific floating ip events by floating ip id.

        :param floating_ip_id: The floating ip id
        :param dict query: Optional query parameters to be sent to limit
                                 the resources being returned.

        :returns: A generator of FloatingIPLog object
        :rtype: :class:`~otcextensions.sdk.anti_ddos.v1.status.FloatingIPLog`
        """
        return self._list(_status.FloatingIPEvent, paginated=False,
                          floating_ip_id=floating_ip_id, **query)

    def floating_ip_stat_day(self, floating_ip_id):
        """List statistic for last 24 hours by floating ip id.

        :param floating_ip_id: The floating ip id

        :returns: A generator of FloatingIPDayStat object
        :rtype:
            :class:`~otcextensions.sdk.anti_ddos.v1.status.FloatingIPDayStat`
        """
        return self._list(_status.FloatingIPDayStat, paginated=False,
                          floating_ip_id=floating_ip_id)

    def floating_ip_stat_week(self, **query):
        """List weekly defence statisticsabout all floating ips.

        :param dict query: Optional query parameters to be sent to limit
            the resources being returned:
            * period_start_date - start day of the 7 day period to query

        :returns: A generator of FloatingIPWeekStat object
        :rtype:
            :class:`~otcextensions.sdk.anti_ddos.v1.status.FloatingIPWeekStat`
        """
        return self._get(_status.FloatingIPWeekStat, requires_id=False,
                         value=None, **query)

    # ======== Project cleanup ========
    def _get_cleanup_dependencies(self):
        return {
            'anti_ddos': {
                'before': ['network']
            }
        }

    def _service_cleanup(self, dry_run=True, client_status_queue=None,
                         identified_resources=None,
                         filters=None, resource_evaluation_fn=None,
                         skip_resources=None):
        pass



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from openstack.tests.unit import base

from otcextensions.sdk.function_graph.v2 import async_notification

EXAMPLE_NOTIFICATION = {
    "func_urn": "urn",
    "max_async_event_age_in_seconds": 60,
    "max_async_retry_attempts": 1,
    "created_at": "2021-03-04T14:50:02+08:00",
    "updated_at": "2021-03-04 14:50:02"
}
EXAMPLE_REQUEST = {
    "request_id": "403fcbd6-ec41-401f-9fa7-386f3d3d****",
    "status": "SUCCESS",
    "error_message": "",
    "start_time": "2019-10-25T15:37:27",
    "end_time": "2019-10-25T15:37:27",
    "error_code": 0
}


class TestFunctionAsyncNotification(base.TestCase):

    def test_basic(self):
        sot = async_notification.Notification()
        path = '/fgs/functions/%(function_urn)s/async-invoke-config'
        self.assertEqual(path, sot.base_path)
        self.assertTrue(sot.allow_create)
        self.assertTrue(sot.allow_list)
        self.assertTrue(sot.allow_delete)
        self.assertTrue(sot.allow_fetch)
        self.assertTrue(sot.allow_commit)

    def test_make_it(self):
        sot = async_notification.Notification(**EXAMPLE_NOTIFICATION)
        self.assertEqual(EXAMPLE_NOTIFICATION['func_urn'], sot.func_urn)
        self.assertEqual(
            EXAMPLE_NOTIFICATION['max_async_event_age_in_seconds'],
            sot.max_async_event_age_in_seconds
        )
        self.assertEqual(
            EXAMPLE_NOTIFICATION['max_async_retry_attempts'],
            sot.max_async_retry_attempts
        )
        self.assertEqual(EXAMPLE_NOTIFICATION['created_at'], sot.created_at)
        self.assertEqual(EXAMPLE_NOTIFICATION['updated_at'], sot.updated_at)


class TestFunctionAsyncRequest(base.TestCase):

    def test_basic(self):
        sot = async_notification.Requests()
        path = '/fgs/functions/%(function_urn)s/async-invocations'
        self.assertEqual(path, sot.base_path)
        self.assertTrue(sot.allow_create)
        self.assertTrue(sot.allow_list)

    def test_make_it(self):
        sot = async_notification.Requests(**EXAMPLE_REQUEST)
        self.assertEqual(EXAMPLE_REQUEST['request_id'], sot.request_id)
        self.assertEqual(EXAMPLE_REQUEST['status'], sot.status)
        self.assertEqual(EXAMPLE_REQUEST['status'], sot.status)
        self.assertEqual(EXAMPLE_REQUEST['start_time'], sot.start_time)
        self.assertEqual(EXAMPLE_REQUEST['end_time'], sot.end_time)
        self.assertEqual(EXAMPLE_REQUEST['error_code'], sot.error_code)



#!/usr/bin/env python3
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
"""
Update CSS Cluster kernel
"""

import openstack

openstack.enable_logging(True)
conn = openstack.connect()

cluster_name_or_id = 'csstest0716'
cluster = conn.css.find_cluster(cluster_name_or_id)

upgrade_type = 'cross'
target_image = conn.css.get_cluster_version_upgrade_info(cluster, upgrade_type)
target_image = next(iter(target_image.image_info_list))
if target_image:
    target_image_id = target_image.id
    indices_backup_check = True
    agency = 'css_upgrade_agency'
    cluster_load_check = True
    conn.css.update_cluster_kernel(
        cluster,
        target_image_id,
        upgrade_type,
        indices_backup_check,
        agency,
        cluster_load_check,
    )



#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.
#
'''LoadBalancer Listener v1 action implementations'''
import logging

from osc_lib import utils
from osc_lib.command import command

from otcextensions.common import sdk_utils
from otcextensions.i18n import _

LOG = logging.getLogger(__name__)


SUPPORTED_PROTOCOLS = ['TCP', 'HTTP', 'HTTPS', 'UDP']

_formatters = {
    'load_balancer_ids': sdk_utils.ListOfIdsColumnBR,
}


def _get_columns(item):
    column_map = {
        'is_admin_state_up': 'admin_state_up',
        'load_balancer_ids': 'loadbalancers',
        # 'listeners': 'listener_ids',
        # 'pools': 'pool_ids',
    }
    return sdk_utils.get_osc_show_columns_for_sdk_resource(item, column_map)


class ListListener(command.Lister):
    _description = _('List listeners')
    column_headers = (
        'id', 'default_pool_id', 'name', 'project_id', 'protocol',
        'protocol_port', 'admin_state_up')
    columns = (
        'id', 'default_pool_id', 'name', 'project_id', 'protocol',
        'protocol_port', 'is_admin_state_up')

    def get_parser(self, prog_name):
        parser = super(ListListener, self).get_parser(prog_name)
        parser.add_argument(
            '--name',
            metavar='<name>',
            help=_('List listeners by listener name.')
        )
        parser.add_argument(
            '--load_balancer',
            metavar='<load_balancer>',
            help=_('Filter by load balancer (name or ID).')
        )
        parser.add_argument(
            '--protocol',
            metavar='{' + ','.join(SUPPORTED_PROTOCOLS) + '}',
            type=lambda s: s.upper(),
            choices=SUPPORTED_PROTOCOLS,
            help=_('Load balancer listener protocol to query\n'
                   'One of [`TCP`, `HTTP`, `HTTPS`, `UDP`]')
        )
        parser.add_argument(
            '--protocol_port',
            type=int,
            metavar='<protocol_port>',
            help=_('Load balancer listener protocol port to query')
        )
        return parser

    def take_action(self, parsed_args):

        args = {}

        if parsed_args.protocol:
            args['protocol'] = parsed_args.protocol
        if parsed_args.protocol_port:
            args['protocol_port'] = parsed_args.protocol_port
        if parsed_args.name:
            args['name'] = parsed_args.name
        # NOTE: loadbalancer_id is not supported in neutron
        if parsed_args.load_balancer:
            args['load_balancer_id'] = parsed_args.load_balancer

        client = self.app.client_manager.network

        data = client.listeners(**args)

        return (
            self.column_headers,
            (utils.get_item_properties(
                s, self.columns, formatters=_formatters
            ) for s in data))


class ShowListener(command.ShowOne):
    _description = _('Show the details of a single listener')

    def get_parser(self, prog_name):
        parser = super(ShowListener, self).get_parser(prog_name)

        parser.add_argument(
            'listener',
            metavar='<listener>',
            help=_('Name or UUID of the listener.')
        )

        return parser

    def take_action(self, parsed_args):

        client = self.app.client_manager.network

        obj = client.find_listener(
            name_or_id=parsed_args.listener,
            ignore_missing=False)

        display_columns, columns = _get_columns(obj)
        data = utils.get_item_properties(obj, columns, formatters=_formatters)

        return (display_columns, data)


class CreateListener(command.ShowOne):
    _description = _('Create LoadBalancer Listener')

    def get_parser(self, prog_name):
        parser = super(CreateListener, self).get_parser(prog_name)

        parser.add_argument(
            'load_balancer',
            metavar='<load_balancer>',
            help=_('Load balancer for the listener (name or ID).')
        )
        parser.add_argument(
            '--name',
            metavar='<name>',
            help=_('Set the listener name.')
        )
        parser.add_argument(
            '--description',
            metavar='<description>',
            help=_('Set the description of this listener.')
        )

        parser.add_argument(
            '--protocol',
            metavar='{' + ','.join(SUPPORTED_PROTOCOLS) + '}',
            type=lambda s: s.upper(),
            choices=SUPPORTED_PROTOCOLS,
            required=True,
            help=_('The protocol for the listener. '
                   'One of [`TCP`, `HTTP`, `HTTPS`, `UDP`]')
        )
        parser.add_argument(
            '--protocol_port',
            type=int,
            metavar='<port>',
            required=True,
            help=_('Set the protocol port number for the listener.')
        )
        parser.add_argument(
            '--connection_limit',
            metavar='<limit>',
            type=int,
            help=_('The maximum number of connections permitted for this '
                   'listener. Default value is -1 which represents infinite '
                   'connections.')
        )
        parser.add_argument(
            '--default_pool',
            metavar='<pool>',
            help=_('The ID of the pool used by the listener if no L7 policies '
                   'match.')
        )
        parser.add_argument(
            '--default_tls_container_ref',
            metavar='<default_tls_container_ref>',
            help=_('The URI of the key manager service secret containing a '
                   'PKCS12 format certificate/key bundle for TERMINATED_TLS '
                   'listeners. DEPRECATED: A secret container of type '
                   '\"certificate\" containing the certificate and key '
                   'for TERMINATED_TLS listeners.')
        )
        admin_group = parser.add_mutually_exclusive_group()
        admin_group.add_argument(
            '--enable',
            action='store_true',
            default=True,
            help=_('Enable listener (default).')
        )
        admin_group.add_argument(
            '--disable',
            action='store_true',
            default=None,
            help=_('Disable listener.')
        )
        return parser

    def take_action(self, parsed_args):

        args = {}

        if parsed_args.load_balancer:
            args['load_balancer_id'] = parsed_args.load_balancer
        if parsed_args.protocol:
            args['protocol'] = parsed_args.protocol
        if parsed_args.protocol_port:
            args['protocol_port'] = parsed_args.protocol_port
        if parsed_args.disable:
            args['is_admin_state_up'] = False
        if parsed_args.connection_limit:
            args['connection_limit'] = parsed_args.connection_limit
        if parsed_args.default_pool:
            args['default_pool_id'] = parsed_args.default_pool
        if parsed_args.default_tls_container_ref:
            args['default_tls_container_ref'] = \
                parsed_args.default_tls_container_ref
        if parsed_args.description:
            args['description'] = parsed_args.description
        if parsed_args.name:
            args['name'] = parsed_args.name

        client = self.app.client_manager.network

        obj = client.create_listener(**args)

        display_columns, columns = _get_columns(obj)
        data = utils.get_item_properties(obj, columns, formatters=_formatters)

        return (display_columns, data)


class SetListener(command.ShowOne):
    _description = _('Update a listener')

    def get_parser(self, prog_name):
        parser = super(SetListener, self).get_parser(prog_name)

        parser.add_argument(
            'listener',
            metavar='<listener>',
            help=_('Listener to modify (ID).')
        )
        parser.add_argument(
            '--name',
            metavar='<name>',
            help=_('Set the listener name.')
        )
        parser.add_argument(
            '--description',
            metavar='<description>',
            help=_('Set the description of this listener.')
        )
        parser.add_argument(
            '--connection_limit',
            metavar='<connection_limit>',
            type=int,
            help=_('The maximum number of connections permitted for this '
                   'listener. Default value is -1 which represents infinite '
                   'connections.')
        )
        parser.add_argument(
            '--default_pool',
            metavar='<pool>',
            help=_('The ID of the pool used by the listener if no L7 policies '
                   'match.')
        )
        parser.add_argument(
            '--default_tls_container_ref',
            metavar='<default_tls_container_ref>',
            help=_('The URI of the key manager service secret containing a '
                   'PKCS12 format certificate/key bundle for TERMINATED_TLS '
                   'listeners. DEPRECATED: A secret container of type '
                   '\"certificate\" containing the certificate and key '
                   'for TERMINATED_TLS listeners.')
        )
        admin_group = parser.add_mutually_exclusive_group()
        admin_group.add_argument(
            '--enable',
            action='store_true',
            default=True,
            help=_('Enable load balancer (default).')
        )
        admin_group.add_argument(
            '--disable',
            action='store_true',
            default=None,
            help=_('Disable load balancer.')
        )

        return parser

    def take_action(self, parsed_args):

        args = {}

        if parsed_args.disable:
            args['is_admin_state_up'] = False
        if parsed_args.connection_limit:
            args['connection_limit'] = parsed_args.connection_limit
        if parsed_args.default_pool:
            args['default_pool_id'] = parsed_args.default_pool
        if parsed_args.default_tls_container_ref:
            args['default_tls_container_ref'] = \
                parsed_args.default_tls_container_ref
        if parsed_args.description:
            args['description'] = parsed_args.description
        if parsed_args.name:
            args['name'] = parsed_args.name

        client = self.app.client_manager.network

        obj = client.update_listener(
            listener=parsed_args.listener,
            **args)

        display_columns, columns = _get_columns(obj)
        data = utils.get_item_properties(obj, columns, formatters=_formatters)

        return (display_columns, data)


class DeleteListener(command.Command):
    _description = _('Delete a listener')

    def get_parser(self, prog_name):
        parser = super(DeleteListener, self).get_parser(prog_name)

        parser.add_argument(
            'listener',
            metavar='<listener>',
            nargs='+',
            help=_('The ID of the listener to delete.')
        )

        return parser

    def take_action(self, parsed_args):

        client = self.app.client_manager.network

        for lsnr in parsed_args.listener:
            obj = client.find_listener(name_or_id=lsnr, ignore_missing=False)
            client.delete_listener(
                listener=obj.id,
                ignore_missing=False)

        return



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
#
import mock
from keystoneauth1 import adapter

from openstack import utils
from openstack.tests.unit import base
from otcextensions.sdk.modelartsv2.v2 import dataset_sample
from otcextensions.tests.unit.sdk.modelartsv2.v2 import examples
from otcextensions.tests.unit.sdk.utils import assert_attributes_equal

EXAMPLE = examples.DATASET_SAMPLE


class TestDatasetSample(base.TestCase):
    def setUp(self):
        super(TestDatasetSample, self).setUp()
        self.sess = mock.Mock(spec=adapter.Adapter)

    def test_basic(self):
        sot = dataset_sample.DatasetSample()

        self.assertEqual(
            "/datasets/%(dataset_id)s/data-annotations/samples",
            sot.base_path,
        )
        self.assertEqual(None, sot.resource_key)
        self.assertEqual("samples", sot.resources_key)

        self.assertTrue(sot.allow_list)
        self.assertTrue(sot.allow_fetch)
        self.assertTrue(sot.allow_create)
        self.assertFalse(sot.allow_delete)
        self.assertFalse(sot.allow_commit)

        self.assertDictEqual(
            {
                "email": "email",
                "high_score": "high_score",
                "label_name": "label_name",
                "label_type": "label_type",
                "limit": "limit",
                "locale": "locale",
                "low_score": "low_score",
                "marker": "marker",
                "offset": "offset",
                "order": "order",
                "preview": "preview",
                "process_parameter": "process_parameter",
                "sample_state": "sample_state",
                "sample_type": "sample_type",
                "search_conditions": "search_conditions",
                "version_id": "version_id",
            },
            sot._query_mapping._mapping,
        )

    def test_make_it(self):
        sot = dataset_sample.DatasetSample(**EXAMPLE)
        self.assertEqual(sot.id, EXAMPLE["sample_id"])
        for key, value in EXAMPLE.items():
            if key == "metadata":
                pass
            else:
                assert_attributes_equal(self, getattr(sot, key), value)

    def test_delete_samples(self):
        dataset_id = "dataset-id"
        samples = ["sample1-id", "sample2-id"]
        sot = dataset_sample.DatasetSample()
        response = mock.Mock()
        response.status_code = 200
        response.json.return_value = {"success": True}
        response.headers = {}
        self.sess.post.return_value = response
        rt = sot.delete_samples(self.sess, dataset_id, samples, True)
        json_body = {
            "samples": ["sample1-id", "sample2-id"],
            "delete_source": True,
        }
        url = utils.urljoin(
            sot.base_path % {"dataset_id": dataset_id}, "delete"
        )
        self.sess.post.assert_called_with(url, json=json_body)
        self.assertEqual(rt, sot)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from openstack import resource
from openstack import exceptions


class StatisticsAPISpec(resource.Resource):
    max_latency = resource.Body('max_latency', type=int)
    avg_latency = resource.Body('avg_latency', type=float)
    req_count = resource.Body('req_count', type=int)
    req_count2xx = resource.Body('req_count_2xx', type=int)
    req_count4xx = resource.Body('req_count_4xx', type=int)
    req_count5xx = resource.Body('req_count_5xx', type=int)
    req_count_error = resource.Body('req_count_error', type=int)
    max_inner_latency = resource.Body('max_inner_latency', type=int)
    avg_inner_latency = resource.Body('avg_inner_latency', type=float)
    max_backend_latency = resource.Body('max_backend_latency', type=int)
    avg_backend_latency = resource.Body('avg_backend_latency', type=float)
    output_throughput = resource.Body('output_throughput', type=float)
    input_throughput = resource.Body('input_throughput', type=float)
    current_minute = resource.Body('current_minute', type=int)
    cycle = resource.Body('cycle')
    api_id = resource.Body('api_id')
    group_id = resource.Body('group_id')
    provider = resource.Body('provider')
    req_time = resource.Body('req_time')
    register_time = resource.Body('register_time')
    status = resource.Body('status', type=int)


class ApiCallResult(resource.Resource):
    allow_fetch = True
    _query_mapping = resource.QueryParameters(
        'api_id', 'duration'
    )

    code = resource.Body('code')
    msg = resource.Body('msg')
    start_time = resource.Body('start_time')
    end_time = resource.Body('end_time')
    api_list = resource.Body('list', type=list,
                             list_type=StatisticsAPISpec)

    def get_api_calls_for_period(self, session, gateway_id, **params):
        """Fetch API calls for a specific period."""
        uri = f'apigw/instances/{gateway_id}/statistics/api/latest'
        response = session.get(uri, params=params)
        exceptions.raise_from_response(response)
        self._translate_response(response)
        return self

    def get_api_calls_for_group(self, session, gateway_id, **attrs):
        """Fetch API calls for a specific group."""
        uri = f'apigw/instances/{gateway_id}/statistics/group/latest'
        response = session.get(uri, params=attrs)
        exceptions.raise_from_response(response)
        self._translate_response(response)
        return self



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from openstack.tests.unit import test_proxy_base

from otcextensions.sdk.vpc.v1 import _proxy
from otcextensions.sdk.vpc.v1 import bandwidth as _bandwidth
from otcextensions.sdk.vpc.v1 import peering
from otcextensions.sdk.vpc.v1 import route
from otcextensions.sdk.vpc.v1 import subnet
from otcextensions.sdk.vpc.v1 import vpc


class TestVpcProxy(test_proxy_base.TestProxyBase):
    def setUp(self):
        super(TestVpcProxy, self).setUp()
        self.proxy = _proxy.Proxy(self.session)


class TestVPCBandwidth(TestVpcProxy):

    def test_assign_bandwidth(self):
        self.verify_create(self.proxy.assign_bandwidth, _bandwidth.Bandwidth,
                           method_kwargs={'name': 'name'},
                           expected_kwargs={
                               'name': 'name',
                               'project_id': self.proxy.get_project_id()
                           })

    def test_bandwidth_delete(self):
        self.verify_delete(
            self.proxy.delete_bandwidth, _bandwidth.Bandwidth, True,
            mock_method='otcextensions.sdk.vpc.v1._proxy.Proxy._delete',
            expected_kwargs={
                'ignore_missing': True,
                'project_id': self.proxy.get_project_id()
            })

    def test_add_eip_to_bandwidth(self):
        bandwidth = _bandwidth.Bandwidth(id='id')
        publicip_info = 'publicip_info'
        version = 'v2.0'
        self._verify(
            'otcextensions.sdk.vpc.v1.bandwidth.Bandwidth.'
            'add_eip_to_bandwidth',
            self.proxy.add_eip_to_bandwidth,
            method_args=[bandwidth, publicip_info],
            expected_args=[self.proxy, version, publicip_info],
            expected_kwargs={'project_id': self.proxy.get_project_id()}
        )

    def test_remove_eip_from_bandwidth(self):
        bandwidth = _bandwidth.Bandwidth(id='id')
        version = 'v2.0'
        project_id = self.proxy.get_project_id()
        self._verify(
            'otcextensions.sdk.vpc.v1.bandwidth.Bandwidth.'
            'remove_eip_from_bandwidth',
            self.proxy.remove_eip_from_bandwidth,
            method_args=[bandwidth],
            expected_args=[self.proxy, project_id, version],
            expected_kwargs={}
        )

    def test_update_bandwidth(self):
        bandwidth = _bandwidth.Bandwidth(id='id')
        version = 'v1'
        project_id = self.proxy.get_project_id()
        self._verify(
            'otcextensions.sdk.vpc.v1.bandwidth.Bandwidth.update_bandwidth',
            self.proxy.update_bandwidth,
            method_args=[bandwidth],
            expected_args=[self.proxy, project_id, version],
            expected_kwargs={}
        )

    def test_find_bandwidth(self):
        version = 'v1'
        base_path = _bandwidth.Bandwidth.base_path % {
            'version': version, 'project_id': self.proxy.get_project_id()}
        self.verify_find(self.proxy.find_bandwidth, _bandwidth.Bandwidth,
                         expected_args=[],
                         expected_kwargs={
                             'project_id': self.proxy.get_project_id(),
                             'base_path': base_path,
                             'ignore_missing': True
                         })


class TestVpcPeering(TestVpcProxy):
    def test_peering_create(self):
        self.verify_create(self.proxy.create_peering, peering.Peering,
                           method_kwargs={'name': 'id'},
                           expected_kwargs={'name': 'id'})

    def test_peering_delete(self):
        self.verify_delete(
            self.proxy.delete_peering,
            peering.Peering, True,
            mock_method='otcextensions.sdk.vpc.v1._proxy.Proxy._delete',
        )

    def test_peering_get(self):
        self.verify_get(self.proxy.get_peering, peering.Peering)

    def test_peerings(self):
        self.verify_list(self.proxy.peerings, peering.Peering)

    def test_peering_update(self):
        self.verify_update(self.proxy.update_peering, peering.Peering)

    def test_set_peering(self):
        self._verify(
            'otcextensions.sdk.vpc.v1.peering.Peering._set_peering',
            self.proxy.set_peering,
            method_args=[peering.Peering],
            method_kwargs={'set_status': 'accept'},
            expected_args=[self.proxy, 'accept']
        )


class TestVpcRoute(TestVpcProxy):
    def test_route_add(self):
        self.verify_create(self.proxy.add_route, route.Route,
                           method_kwargs={'name': 'id'},
                           expected_kwargs={'name': 'id'})

    def test_route_delete(self):
        self.verify_delete(
            self.proxy.delete_route,
            route.Route, True,
            mock_method='otcextensions.sdk.vpc.v1._proxy.Proxy._delete',
        )

    def test_route_get(self):
        self.verify_get(self.proxy.get_route, route.Route)

    def test_routes(self):
        self.verify_list(self.proxy.routes, route.Route)


class TestVpc(TestVpcProxy):
    def test_vpc_create(self):
        self.verify_create(self.proxy.create_vpc, vpc.Vpc,
                           method_kwargs={'name': 'id'},
                           expected_kwargs={
                               'name': 'id',
                               'project_id': self.proxy.get_project_id()
                           })

    def test_vpc_delete(self):
        self.verify_delete(
            self.proxy.delete_vpc, vpc.Vpc, True,
            mock_method='otcextensions.sdk.vpc.v1._proxy.Proxy._delete',
            expected_kwargs={
                'ignore_missing': True,
                'project_id': self.proxy.get_project_id()
            })

    def test_vpc_get(self):
        self.verify_get(self.proxy.get_vpc, vpc.Vpc, 'id',
                        expected_kwargs={
                            'project_id': self.proxy.get_project_id()})

    def test_vpcs(self):
        self.verify_list(self.proxy.vpcs, vpc.Vpc,
                         expected_kwargs={
                             'project_id': self.proxy.get_project_id()
                         })

    def test_vpc_update(self):
        self.verify_update(self.proxy.update_vpc, vpc.Vpc,
                           method_kwargs={'name': 'id'},
                           expected_kwargs={
                               'name': 'id',
                               'project_id': self.proxy.get_project_id()
                           })

    def test_vpc_find(self):
        self.verify_find(self.proxy.find_vpc, vpc.Vpc, 'id',
                         expected_kwargs={
                             'project_id': self.proxy.get_project_id()
                         })


class TestSubnet(TestVpcProxy):
    def test_subnet_create(self):
        self.verify_create(self.proxy.create_subnet, subnet.Subnet,
                           method_kwargs={'name': 'id'},
                           expected_kwargs={
                               'name': 'id',
                               'project_id': self.proxy.get_project_id()
                           })

    def test_subnets(self):
        self.verify_list(self.proxy.subnets, subnet.Subnet,
                         expected_kwargs={
                             'project_id': self.proxy.get_project_id()
                         })

    def test_subnet_find(self):
        self.verify_find(self.proxy.find_subnet, subnet.Subnet, 'id',
                         expected_kwargs={
                             'project_id': self.proxy.get_project_id()
                         })

    def test_subnet_get(self):
        self.verify_get(self.proxy.get_subnet, subnet.Subnet, 'id',
                        expected_kwargs={
                            'project_id': self.proxy.get_project_id(),
                        })

    def test_subnet_update(self):
        self.verify_update(self.proxy.update_subnet, subnet.Subnet,
                           method_kwargs={'vpc_id': 'vpc'},
                           expected_kwargs={
                               'base_path': subnet.vpc_subnet_base_path('vpc'),
                               'project_id': self.proxy.get_project_id(),
                           })

    def test_subnet_delete(self):
        self.verify_delete(
            self.proxy.delete_subnet, subnet.Subnet, True,
            mock_method='otcextensions.sdk.vpc.v1._proxy.Proxy._delete',
            method_kwargs={'vpc_id': 'vpc'},
            expected_kwargs={
                'ignore_missing': True,
                'base_path': subnet.vpc_subnet_base_path('vpc'),
            })


class TestExtractName(TestVpcProxy):

    def test_extract_name(self):
        self.assertEqual(
            ['vpcs'],
            self.proxy._extract_name('/v1/123/vpcs', project_id='123')
        )
        self.assertEqual(
            [],
            self.proxy._extract_name(
                '/', project_id='123')
        )



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from openstack.tests.unit import base

from otcextensions.sdk.vlb.v3 import l7_policy

EXAMPLE = {
    'action': 'action',
    'created_at': 'created-at',
    'description': 'description',
    'admin_state_up': True,
    'listener_id': 'listener-id',
    'name': 'name',
    'priority': 1,
    'project_id': 'project-id',
    'provisioning_status': 'provisioning-status',
    'redirect_pool_id': 'redirect-pool-id',
    'position': 1,
    'redirect_listener_id': 'redirect-listener-id',
    'redirect_url': 'redirect-url',
    'rules': [],
    'redirect_url_config': {},
    'fixed_response_config': {},
    'updated_at': 'updated-at',
}


class TestLoadBalancer(base.TestCase):

    def test_basic(self):
        sot = l7_policy.L7Policy()
        path = '/elb/l7policies'
        self.assertEqual(path, sot.base_path)
        self.assertTrue(sot.allow_create)
        self.assertTrue(sot.allow_list)
        self.assertTrue(sot.allow_fetch)
        self.assertTrue(sot.allow_commit)
        self.assertTrue(sot.allow_delete)

    def test_make_it(self):
        sot = l7_policy.L7Policy(**EXAMPLE)
        self.assertEqual(EXAMPLE['action'], sot.action)
        self.assertEqual(EXAMPLE['created_at'], sot.created_at)
        self.assertEqual(
            EXAMPLE['description'],
            sot.description)
        self.assertEqual(EXAMPLE['admin_state_up'], sot.is_admin_state_up)
        self.assertEqual(EXAMPLE['listener_id'], sot.listener_id)
        self.assertEqual(EXAMPLE['name'], sot.name)
        self.assertEqual(EXAMPLE['priority'], sot.priority)
        self.assertEqual(EXAMPLE['project_id'], sot.project_id)
        self.assertEqual(
            EXAMPLE['provisioning_status'],
            sot.provisioning_status)
        self.assertEqual(EXAMPLE['redirect_pool_id'], sot.redirect_pool_id)
        self.assertEqual(EXAMPLE['position'], sot.position)
        self.assertEqual(EXAMPLE['redirect_listener_id'],
                         sot.redirect_listener_id)
        self.assertEqual(EXAMPLE['redirect_url'], sot.redirect_url)
        self.assertEqual(EXAMPLE['rules'], sot.rules)
        self.assertEqual(EXAMPLE['redirect_url_config'],
                         sot.redirect_url_config)
        self.assertEqual(EXAMPLE['fixed_response_config'],
                         sot.fixed_response_config)
        self.assertEqual(EXAMPLE['updated_at'], sot.updated_at)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from openstack import proxy

from otcextensions.common.utils import extract_url_parts
from otcextensions.sdk.waf.v1 import certificate as _cert
from otcextensions.sdk.waf.v1 import domain as _domain


class Proxy(proxy.Proxy):
    skip_discovery = True

    def _extract_name(self, url, service_type=None, project_id=None):
        return extract_url_parts(url, project_id)

    def __init__(self, session, *args, **kwargs):
        super(Proxy, self).__init__(session=session, *args, **kwargs)
        self.endpoint_override = \
            '%s/%s/%s' % (
                self.get_endpoint(),
                'v1',
                '%(project_id)s')
        self.additional_headers = {
            'x-request-source-type': 'ApiCall',
            'content-type': 'application/json'
        }

    # ======== Certificates ========
    def certificates(self, **query):
        """Retrieve a generator of certificates

        :param dict query: Optional query parameters to be sent to limit the
            resources being returned.
            * `limit`: pagination limit
            * `offset`: pagination offset

        :returns: A generator of certificate
            :class:`~otcextensions.sdk.waf.v1.certificate.Certificate`
            instances
        """
        return self._list(_cert.Certificate, **query)

    def create_certificate(self, **attrs):
        """Upload certificate from attributes

        :param dict attrs: Keyword arguments which will be used to create
            a :class:`~otcextensions.sdk.waf.v1.certificate.Certificate`,
            comprised of the properties on the Certificate class.
        :returns: The results of certificate creation
        :rtype: :class:`~otcextensions.sdk.waf.v1.certificate.Certificate`
        """
        return self._create(_cert.Certificate, prepend_key=False, **attrs)

    def get_certificate(self, certificate):
        """Get a certificate

        :param certificate: The value can be the ID of a certificate
             or a :class:`~otcextensions.sdk.waf.v1.certificate.Certificate`
             instance.
        :returns: Certificate instance
        :rtype: :class:`~otcextensions.sdk.waf.v1.certificate.Certificate`
        """
        return self._get(_cert.Certificate, certificate)

    def delete_certificate(self, certificate, ignore_missing=True):
        """Delete a certificate

        :param certificate: The value can be the ID of a certificate
             or a :class:`~otcextensions.sdk.waf.v1.certificate.Certificate`
             instance.
        :param bool ignore_missing: When set to ``False``
            :class:`~openstack.exceptions.ResourceNotFound` will be raised when
            the certificate does not exist.
            When set to ``True``, no exception will be set when attempting to
            delete a nonexistent certificate.

        :returns: Certificate been deleted
        :rtype: :class:`~otcextensions.sdk.waf.v1.certificate.Certificate`
        """
        return self._delete(_cert.Certificate, certificate,
                            ignore_missing=ignore_missing)

    def update_certificate(self, certificate, **attrs):
        """Update certificate attributes

        :param certificate: The id or an instance of
            :class:`~otcextensions.sdk.waf.v1.certificate.Certificate`
        :param dict attrs: attributes for update on
            :class:`~otcextensions.sdk.waf.v1.certificate.Certificate`

        :rtype: :class:`~otcextensions.sdk.waf.v1.certificate.Certificate`
        """
        return self._update(_cert.Certificate, certificate, **attrs)

    def find_certificate(self, name_or_id, ignore_missing=True, **attrs):
        """Find a single certificate

        :param name_or_id: The name or ID of a certificate
        :param bool ignore_missing: When set to ``False``
            :class:`~openstack.exceptions.ResourceNotFound` will be raised
            when the certificate does not exist.
            When set to ``True``, no exception will be set when attempting
            to delete a nonexistent certificate.

        :returns: ``None``
        """
        return self._find(_cert.Certificate, name_or_id,
                          ignore_missing=ignore_missing,
                          **attrs)

    # ======== Domains ========
    def domains(self, **query):
        """Retrieve a generator of domains

        :param dict query: Optional query parameters to be sent to limit the
            resources being returned.
            * `limit`: pagination limit
            * `offset`: pagination offset
            * `name`: domain name (hostname)
            * `policy_name`: policy name

        :returns: A generator of domain
            :class:`~otcextensions.sdk.waf.v1.domain.Domain`
            instances
        """
        return self._list(_domain.Domain, **query)

    def create_domain(self, **attrs):
        """Upload domain from attributes

        :param dict attrs: Keyword arguments which will be used to create
            a :class:`~otcextensions.sdk.waf.v1.domain.Domain`,
            comprised of the properties on the Domain class.
        :returns: The results of domain creation
        :rtype: :class:`~otcextensions.sdk.waf.v1.domain.Domain`
        """
        return self._create(_domain.Domain, prepend_key=False, **attrs)

    def get_domain(self, domain):
        """Get a domain

        :param domain: The value can be the ID of a domain
             or a :class:`~otcextensions.sdk.waf.v1.domain.Domain`
             instance.
        :returns: Domain instance
        :rtype: :class:`~otcextensions.sdk.waf.v1.domain.Domain`
        """
        return self._get(_domain.Domain, domain)

    def delete_domain(self, domain, ignore_missing=True):
        """Delete a domain

        :param domain: The value can be the ID of a domain
             or a :class:`~otcextensions.sdk.waf.v1.domain.Domain`
             instance.
        :param bool ignore_missing: When set to ``False``
            :class:`~openstack.exceptions.ResourceNotFound` will be raised when
            the domain does not exist.
            When set to ``True``, no exception will be set when attempting to
            delete a nonexistent domain.

        :returns: Domain been deleted
        :rtype: :class:`~otcextensions.sdk.waf.v1.domain.Domain`
        """
        return self._delete(_domain.Domain, domain,
                            ignore_missing=ignore_missing)

    def update_domain(self, domain, **attrs):
        """Update domain attributes

        :param domain: The id or an instance of
            :class:`~otcextensions.sdk.waf.v1.domain.Domain`
        :param dict attrs: attributes for update on
            :class:`~otcextensions.sdk.waf.v1.domain.Domain`

        :rtype: :class:`~otcextensions.sdk.waf.v1.domain.Domain`
        """
        return self._update(_domain.Domain, domain, **attrs)

    def find_domain(self, name_or_id, ignore_missing=True, **attrs):
        """Find a single domain

        :param name_or_id: The name or ID of a domain
        :param bool ignore_missing: When set to ``False``
            :class:`~openstack.exceptions.ResourceNotFound` will be raised
            when the domain does not exist.
            When set to ``True``, no exception will be set when attempting
            to delete a nonexistent domain.

        :returns: ``None``
        """
        return self._find(_domain.Domain, name_or_id,
                          ignore_missing=ignore_missing,
                          **attrs)

    # ======== Project cleanup ========
    def _get_cleanup_dependencies(self):
        return {
            'waf': {
                'before': []
            }
        }

    def _service_cleanup(self, dry_run=True, client_status_queue=None,
                         identified_resources=None,
                         filters=None, resource_evaluation_fn=None,
                         skip_resources=None):
        for obj in self.domains():
            self._service_cleanup_del_res(
                self.delete_domain,
                obj,
                dry_run=dry_run,
                client_status_queue=client_status_queue,
                identified_resources=identified_resources,
                filters=filters,
                resource_evaluation_fn=resource_evaluation_fn)

        for obj in self.certificates():
            self._service_cleanup_del_res(
                self.delete_certificate,
                obj,
                dry_run=dry_run,
                client_status_queue=client_status_queue,
                identified_resources=identified_resources,
                filters=filters,
                resource_evaluation_fn=resource_evaluation_fn)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from openstack.tests.unit import base
from otcextensions.sdk.apig.v2 import az


EXAMPLE_AZ = {
    'name': 'eu-de-1a',
    'id': 'az-12345',
    'code': 'eu-de-1a',
    'port': 443,
    'local_name': {'en_us': 'Europe Germany AZ1', 'zh_cn': '欧洲德国AZ1'},
    'specs': {'max_bandwidth': 1000, 'min_bandwidth': 50}
}


class TestAZ(base.TestCase):

    def test_basic(self):
        sot = az.AZ()
        self.assertEqual('/apigw/available-zones', sot.base_path)
        self.assertTrue(sot.allow_list)
        self.assertEqual('AZ', sot.resource_name)
        self.assertEqual('available_zones', sot.resources_key)

    def test_make_it(self):
        sot = az.AZ(**EXAMPLE_AZ)
        self.assertEqual(EXAMPLE_AZ['name'], sot.name)
        self.assertEqual(EXAMPLE_AZ['id'], sot.id)
        self.assertEqual(EXAMPLE_AZ['code'], sot.code)
        self.assertEqual(EXAMPLE_AZ['port'], sot.port)
        self.assertEqual(EXAMPLE_AZ['local_name']['en_us'],
                         sot.local_name.en_us)
        self.assertEqual(EXAMPLE_AZ['local_name']['zh_cn'],
                         sot.local_name.zh_cn)
        self.assertEqual(EXAMPLE_AZ['specs'], sot.specs)
